<?php

namespace App\Exports;

use App\Models\Product;
use Maatwebsite\Excel\Concerns\FromQuery;
use Maatwebsite\Excel\Concerns\Exportable;
use DB;

class ProductsExport implements FromQuery
{
    use Exportable;

    public function __construct($criteria)
    {
        $this->criteria = $criteria;
    }
    public function query()
    {
        $keyword = $this->criteria->keyword;
        $id = $this->criteria->id;
        $start_date = $this->criteria->start_date;
        $end_date = $this->criteria->end_date;
        $filter = $this->criteria->type;
        $show_bmi = isset($this->criteria->show_bmi) ? $this->criteria->show_bmi: false;

        $detail = Product::query()->select('products.title',  DB::raw('sum(d.quantity) as quantity'))
        ->join('order_details as d', 'd.product_id', 'products.id')
        ->join('orders as o', 'o.id', 'd.order_id')
        ->where('o.payment_status', 0)
        ->where('o.order_status', '!=', 5)
        ->where('o.order_from', 1);
        if ($keyword != NULL && $keyword != '') {
            $names = strtolower($keyword);
            $detail->where('products.title', 'like', "%{$names}%");
        }   
        if ($start_date != NULL && $end_date != NULL) {
            $detail->whereDate('o.created_at', '>=', $start_date)
                ->whereDate('o.created_at', '<=', $end_date);
        } elseif ($start_date != NULL && $end_date == NULL) {
            $detail->whereDate('o.created_at', '=', $start_date);
        } elseif ($start_date == NULL && $end_date != NULL) {
            $detail->whereDate('o.created_at', '=', $end_date);
        }
        if($show_bmi)
        {
            $detail->whereHas('categories', function($query) {
                $query->where('product_categories.id', 103);
            });
        }
       
        $detail->groupBy('products.id')->orderBy('quantity','desc');
        return $detail;
    }
}