<?php

namespace App\Helpers;

use App\Models\Customer\Customer;
use App\Models\Order;
use App\Models\Order\Cart;
use Carbon\Carbon;
 
use App\Models\Order\OrderDetail;
use App\Models\Product\OtcProduct;
use App\Models\Product\Product;
use DB;
use Illuminate\Support\Facades\Auth;

class OrderHelper
{

    public static function last_year_order_count($customer_id, $order_id, $order_date)
    {
        $last12months = Carbon::parse($order_date)->submonths(12)->format('Y-m-d');

        $orders = Order::join('order_customers', 'orders.id', 'order_customers.order_id')
            ->join('order_details', 'orders.id', 'order_details.order_id')
            ->where('order_customers.customer_id', $customer_id)
            ->where('orders.order_status', '=', 4)
            ->where('order_details.order_id', '<>', $order_id)
            ->where('orders.created_at', '>', $last12months)
            ->where('orders.created_at', '<', $order_date)
            ->groupBy('orders.id')
            ->get('orders.id');

        if ($orders) {
            return $orders->count();
        } else {
            return 0;
        }
    }

    public static function last_order_date($customer_id, $order_id, $product_id, $order_date)
    {

        $ordered_date = OrderDetail::select([DB::raw('date(order_details.created_at) as created_at')])
            ->join('products', 'order_details.product_id', 'products.id')
            ->join('orders', 'orders.id', 'order_details.order_id')
            ->join('order_customers', 'order_customers.order_id', 'order_details.order_id')
            ->where('order_details.product_id', '=', $product_id)
            ->where('order_details.order_id', '<>', $order_id)
            ->where('order_customers.customer_id', '=', $customer_id)
            ->where('orders.order_status', '=', 4)
            ->where('orders.created_at', '<', $order_date)
            ->orderby('order_details.id', 'desc')
            ->first();
        if ($ordered_date) {
            $dbdte = $ordered_date->created_at;
            $today = $dbdte->format('d-m-Y');
            return $today;
        } else {
            $today = '***NA***';
            return $today;
        }
    }

    public static function update_pvt_prescription_order($order_id)
    {
        $order = Order::find($order_id);
        $prescriptions = $order->prescriptions;
        if ($prescriptions) {
            $prescription = $prescriptions->first();
            $prescription->status = 3;
            $prescription->save();
        }
        if ($order->order_type == 4) {
            $items = $order->items;
            foreach ($items as $item) {
                $product = Product::find($item->product_id);
                if ($product) {
                    $product->delete();
                }
            }
        }
    }

    public static function high_risk_last_order($customer_id, $product_id)
    {
        $last_order = "";
        $ordered_date = OrderDetail::select([DB::raw('date(order_details.created_at) as created_at')])
            ->join('products', 'order_details.product_id', 'products.id')
            ->join('orders', 'orders.id', 'order_details.order_id')
            ->join('order_customers', 'order_customers.order_id', 'order_details.order_id')
            ->where('order_details.product_id', '=', $product_id)
            ->where('order_customers.customer_id', '=', $customer_id)
            ->where('orders.order_status', '=', 4)
            ->orderby('order_details.id', 'desc')
            ->first();
        if ($ordered_date) {
            $last_order = $ordered_date->created_at;
        }
        return $last_order;
    }

    public static function is_bmi_order($order_id)
    {
        $bmi_order = Order::join('order_details', 'orders.id', 'order_details.order_id')
            ->join('products', 'order_details.product_id', 'products.id')
            ->leftJoin('category_product as parent_category', 'parent_category.product_id', '=', 'products.parent_id')
            ->leftJoin('category_product', 'category_product.product_id', 'products.id')
            ->where(function ($query) {
                $query->where('category_product.category_id', 103)
                    ->orWhere('parent_category.category_id', 103);
            })
            ->where('orders.id', $order_id)
            ->first();
        if ($bmi_order) {
            return true;
        }
        return false;
    }

    public static function is_valid_cart($session_id, $product, $ctitle)
    {
        $acart = Cart::where('title', $ctitle)
            ->where('session_id', $session_id)
            ->first();
        if ($acart) {
            $message = 'You can buy only one unit of this product.';
            $return = ['error' => true, 'message' => $message];
            return $return;
        }
        if ($product->is_otc) {
            if ($product->parent_id == 0)
                $otc_product = $product->otc_product;
            else {
                $parent_product = Product::find($product->parent_id);
                $otc_product = $parent_product->otc_product;
            }
            if ($otc_product && ($otc_product->promethazine == 1 || $otc_product->codeine == 1 || $otc_product->pseudoephedrine == 1)) {
                $carts = Cart::where('session_id', $session_id)
                    ->where('is_otc', 1)
                    ->get();
                foreach ($carts as $cart) {
                    if ($otc_product->promethazine == 1) {
                        $cart_otc = OtcProduct::where('product_id', $cart->product_id)
                            ->where('promethazine', 1)->first();
                        if ($cart_otc) {
                            $message = 'You cannot purchase more than 1 Promethazine containing product';
                            $return = ['error' => true, 'message' => $message];
                            return $return;
                        }
                    } elseif ($otc_product->codeine == 1) {
                        $cart_otc = OtcProduct::where('product_id', $cart->product_id)
                            ->where('codeine', 1)->first();
                        if ($cart_otc) {
                            $message = 'You cannot purchase more than 1 Codeine containing product';
                            $return = ['error' => true, 'message' => $message];
                            return $return;
                        }
                    } elseif ($otc_product->pseudoephedrine == 1) {
                        $cart_otc = OtcProduct::where('product_id', $cart->product_id)
                            ->where('pseudoephedrine', 1)->first();
                        if ($cart_otc) {
                            $message = 'You cannot purchase more than 1 Pseudoephedrine containing product';
                            $return = ['error' => true, 'message' => $message];
                            return $return;
                        }
                    }
                }
            }
        }
        $return = ['error' => false];
        return $return;
    }

    public static function update_query_order($order_id)
    {
        OrderDetail::where('order_id', $order_id)
            ->update(['consultation_type' => 1]);
    }

    public static function can_reorder($order_id)
    {
        $return = true;
        $order = Order::find($order_id);
        $products = $order->products;
        foreach ($products as $product) {
            if ($product->related_id != 0 && $product->question_group_id == NULL) {
                $product = Product::find($product->related_id);
            }
            if ($product->question_group_id != NULL) {
                $return = false;
            }
        }
        return $return;
    }
    public static function view_private_orders()
    {
        $return = false;
        $user = Auth::user();
        $customer = Customer::where('user_id', $user->id)->first();
        if ($customer && $customer->private_orders->count() > 0) {
            $return = true;
        }
        return $return;
    }
}
