<?php

namespace App\Http\Controllers;

use App\Models\Page;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class BrandController extends Controller
{
    protected $title = 'Brands';
    protected $viewPath = 'brands';
    protected $link = 'brands';

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    /**
     * Show the Dashboard.
     */
    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'title-desc' => 'Brand Name Desc',
            'title-asc' => 'Brand Name Asc',
            'quantity-desc' => 'Products Sold Desc',
            'quantity-asc' => 'Products Sold Asc',
        ];

        $filter = $request->filter;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath, compact('page', 'request'));
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function lists(Request $request)
    {
        $sort = ['quantity', 'desc'];
        $detail = DB::table('orders as o')
            ->select('b.id', 'b.title', DB::raw('sum(oc.qty) as quantity'))
            ->join('order_carts as oc', 'o.id', 'oc.order_id')
            ->join('contents as c', 'oc.product_id', 'c.id')
            ->join('products as p', 'p.content_id', 'c.id')
            ->join('contents as b', 'p.brand', 'b.id')
            ->where('o.payment', 1)
            ->where('p.brand', '!=', 0);

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('b.title', 'like', "%{$names}%");
                }

                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereBetween('o.datetime', [$from_date, $to_date]);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.datetime', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.datetime', '=', $to_date);
            }
        }
        $detail->groupBy('b.id', 'b.title')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->removeColumn('id')
            ->escapeColumns([])
            ->make(false);
    }
}
