<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Order;
use App\Models\OrderDetail;
use App\Models\Page;
use App\Models\Product;
use App\Models\ProductCategory;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;

use Illuminate\Support\Facades\Response;


/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class CategoryController extends Controller
{
    protected $title = 'Categories';
    protected $viewPath = 'categories';
    protected $link = 'categories';

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    /**
     * Show the Dashboard.
     */
    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'product_categories.title-desc' => 'Category Name Desc',
            'product_categories.title-asc' => 'Category Name Asc',
            'quantity-desc' => 'Products Sold Desc',
            'quantity-asc' => 'Products Sold Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/lists', compact('page', 'request'));
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function lists(Request $request)
    {
        $sort = ['quantity', 'desc'];
        $detail = ProductCategory::select('product_categories.id', 'product_categories.title', DB::raw('sum(d.quantity) as quantity'))
            ->join('category_product as cp', 'cp.category_id', 'product_categories.id')
            ->join('order_details as d', 'd.product_id', 'cp.product_id')
            ->join('orders as o', 'o.id', 'd.order_id')
            ->where('o.payment_status', 0)
            ->where('o.order_status', '=', 4)
            ->where('o.order_from', 1);
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('product_categories.title', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
        }
        $detail->groupBy('product_categories.id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->addColumn('action', function ($detail) {
                $product_url = url('categories/' . $detail->id . '/products');
                $salet_url = url('categories/' . $detail->id . '/sales');

                return HtmlHelper::showButton($product_url, 'Product Detail', 'cube').''.HtmlHelper::showButton($salet_url, 'Sales', 'eye');
            })
            ->removeColumn('id')
            ->escapeColumns([])
            ->make(false);
    }

    public function salesIndex(Request $request, $id)
    {
        $category = ProductCategory::findOrFail($id);
        $page = collect();
        $page->title = $category->title.' Sales';
        $page->link = url('categories/' . $id . '/sales');
        $page->sorts = [
            'quantity-desc' => 'Quantity Desc',
            'quantity-asc' => 'Quantity Asc',
            'products.title-desc' => 'Product Name Desc',
            'products.title-asc' => 'Product Name Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->id = $category->id ?? '';
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';
        return view($this->viewPath . '/sales', compact('page', 'request'));
    }


        /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function salesLists(Request $request, $id)
    {
        $sort = ['quantity', 'desc'];




        $detail = OrderDetail::select(
            'c.first_name','c.last_name','c.email'
            ,'order_details.quantity',
            'order_details.title as product_name','o.order_no','o.created_at'
        )
            ->join('category_product as cp', 'cp.product_id','order_details.product_id')
            ->join('orders as o', 'o.id', 'order_details.order_id')
            ->join('order_customers as oc', 'oc.order_id', 'o.id')
            ->join('customers as c', 'c.id', 'oc.customer_id')


            ->where('cp.category_id', $id)
            ->where('o.payment_status', 0)
            ->where('o.order_status', '!=', 5)
            ->where('o.order_from', 1);
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('order_details.title', 'like', "%{$names}%");
                    $detail->orWhere('c.first_name', 'like', "%{$names}%");
                    $detail->orWhere('c.last_name', 'like', "%{$names}%");
                    $detail->orWhere('c.email', 'like', "%{$names}%");
                }

                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
        }
        $detail->groupBy('c.id')
            ->orderBy($sort[0], $sort[1]);

            
        return DataTables::of($detail)
            ->addColumn('action', function ($detail) {
                $product_url = url('products/' . $detail->id . '/' . $detail->slug);
                return HtmlHelper::showButton($product_url, 'Order Detail', 'link');
            })
            ->removeColumn('id', 'slug')
            ->escapeColumns([])
            ->make(false);
    }

    public function salesExport(Request $request, $id)
    {
     



        $detail = OrderDetail::select(
            'c.first_name','c.last_name','c.email'
            ,'order_details.quantity',
            'order_details.title as product_name','o.order_no','o.created_at'
        )
            ->join('category_product as cp', 'cp.product_id','order_details.product_id')
            ->join('orders as o', 'o.id', 'order_details.order_id')
            ->join('order_customers as oc', 'oc.order_id', 'o.id')
            ->join('customers as c', 'c.id', 'oc.customer_id')


            ->where('cp.category_id', $id)
            ->where('o.payment_status', 0)
            ->where('o.order_status', '!=', 5)
            ->where('o.order_from', 1);
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('order_details.title', 'like', "%{$names}%");
                    $detail->orWhere('c.first_name', 'like', "%{$names}%");
                    $detail->orWhere('c.last_name', 'like', "%{$names}%");
                    $detail->orWhere('c.email', 'like', "%{$names}%");
                }

                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
        }
        $detail= $detail->groupBy('c.id')->get();

     
                  
	  $columns = array('#','first_name','last_name','email','quantity','product_name','order_no','created_at');
	 
            
	  
      $file = 'Category_orders.csv';   
      $headers = array(
        "Content-type" => "text/csv",
        "Content-Disposition" => "attachment; filename=". $file,
        "Pragma" => "no-cache",
        "Cache-Control" => "must-revalidate, post-check=0, pre-check=0",
        "Expires" => "0"
      );

	    
      $callback = function() use ($columns,$detail)
      {
        $file = fopen('php://output', 'w');
        fputcsv($file, $columns);
       
		$nextindex = 0;
		$productcount = 0;
        foreach($detail as $key => $orders) {
		
			$nextkey = $key + 1;
			$nextindex = $nextindex + 1;
            
			$nextorderid = 0;
			if(isset($order_detail[$nextkey])){
			  $nextorderid = $order_detail[$nextkey]->id;
			}
			
			fputcsv($file, array($nextindex,$orders->first_name,$orders->last_name,$orders->email,$orders->quantity,$orders->product_name,$orders->order_no,$orders->created_at));
            
        }
        fclose($file);
      };

      return Response::stream($callback, 200, $headers);

    }



    public function productIndex(Request $request, $id)
    {
        $category = ProductCategory::findOrFail($id);
        $page = collect();
        $page->title = 'Category - ' . $category->title;
        $page->link = url('categories/' . $id . '/products');
        $page->sorts = [
            'quantity-desc' => 'Quantity Desc',
            'quantity-asc' => 'Quantity Asc',
            'products.title-desc' => 'Product Name Desc',
            'products.title-asc' => 'Product Name Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->id = $category->id ?? '';
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';
        return view($this->viewPath . '/products', compact('page', 'request'));
    }
    

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function productLists(Request $request, $id)
    {
        $sort = ['quantity', 'desc'];
        $detail = Product::select('products.id', 'products.title as product', 'products.slug', DB::raw('sum(d.quantity) as quantity'))
            ->join('category_product as cp', 'cp.product_id', 'products.id')
            ->join('order_details as d', 'd.product_id', 'products.id')
            ->join('orders as o', 'o.id', 'd.order_id')
            ->where('cp.category_id', $id)
            ->where('o.payment_status', 0)
            ->where('o.order_status', '!=', 5)
            ->where('o.order_from', 1);
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('products.title', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
        }
        $detail->groupBy('d.product_id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->addColumn('action', function ($detail) {
                $product_url = url('products/' . $detail->id . '/' . $detail->slug);
                return HtmlHelper::showButton($product_url, 'Order Detail', 'link');
            })
            ->removeColumn('id', 'slug')
            ->escapeColumns([])
            ->make(false);
    }

    public function salesDataDashboard(Request $request)
    {
        $filter = $request->filter;
        $start_date = '';
        $end_date = '';
        if ($filter != '') {
            $start_date = Carbon::now();
            if ($filter == 'lweek') {
                $end_date = Carbon::now()->subWeek();
            } elseif ($filter == 'lmonth') {
                $end_date = Carbon::now()->subMonth();
            } elseif ($filter == 'lyear') {
                $end_date = Carbon::now()->subYear();
            }
        }

        $criteria = collect();
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        return Order::getCategories($criteria);
    }


}
