<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Order;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class CouponController extends Controller
{
    protected $title = 'Coupons';
    protected $viewPath = 'coupons';
    protected $link = 'coupons';

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'counts-desc' => 'Coupon Used Desc',
            'counts-asc' => 'Coupon Used Asc',
            'coupon-desc' => 'Coupon Code Desc',
            'coupon-asc' => 'Coupon Code Asc',
        ];
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/lists', compact('page', 'request'));
    }

    public function lists(Request $request)
    {
        $sort = ['counts', 'desc'];
        $detail = Order::select('coupon', DB::raw('count(id) as counts'))
            ->whereNotNull('coupon')
            ->paid()->online();
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('coupon', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '>=', $from_date)
                    ->whereDate('created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '=', $to_date);
            }
        }
        $detail->groupBy('coupon')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->addColumn('action', function ($detail) {
                $product_url = url('coupons/' . $detail->coupon);
                return HtmlHelper::showButton($product_url, 'Coupon Detail', 'link');
            })
            ->removeColumn()
            ->escapeColumns([])
            ->make(false);
    }

    public function couponIndex(Request $request, $coupon)
    {
        $page = collect();
        $page->title = 'Coupon - ' . $coupon;
        $page->link = url('coupons/' . $coupon);
        $page->sorts = [
            'orders.created_at-desc' => 'Latest Order',
            'orders.created_at-asc' => 'Oldest Order',
            'orders.id-desc' => 'Order ID Desc',
            'orders.id-asc' => 'Order ID Asc',
            'orders.billing_first_name-desc' => 'Customer Name Desc',
            'orders.billing_first_name-asc' => 'Customer Name Asc',
            'orders.billing_city-desc' => 'Location Desc',
            'orders.billing_city-asc' => 'Location Asc'
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->id = $product->id ?? '';
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/coupon', compact('page', 'request'));
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function couponLists(Request $request, $coupon)
    {
        $sort = ['orders.created_at', 'desc'];
        $detail = Order::select('id', 'created_at', 'billing_first_name', 'billing_last_name', 'billing_postcode', 'billing_city')
            ->where('coupon', $coupon)
            ->paid()->online();

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('billing_first_name', 'like', "%{$names}%")
                            ->orWhere('id', 'like', "%{$names}%")
                            ->orWhere('billing_last_name', 'like', "%{$names}%")
                            ->orWhere('billing_postcode', 'like', "%{$names}%")
                            ->orWhere('billing_city', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '>=', $from_date)
                    ->whereDate('created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('created_at', '=', $to_date);
            }
        }
        $detail->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('name', function ($detail) {
                return $detail->billing_first_name . ' ' . $detail->billing_last_name;
            })
            ->editColumn('postcode', function ($detail) {
                return $detail->billing_city . ', ' . $detail->billing_postcode;
            })
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })
            ->addColumn('action', function ($detail) {
                $product_url = url('sales/' . $detail->id);
                return HtmlHelper::showButton($product_url, 'Order Detail', 'link');
            })
            ->removeColumn('billing_last_name', 'billing_city')
            ->escapeColumns([])
            ->make(false);
    }

    public function salesDataDashboard(Request $request)
    {
        $filter = $request->filter;
        $start_date = '';
        $end_date = '';
        if ($filter != '') {
            $start_date = Carbon::now();
            if ($filter == 'lweek') {
                $end_date = Carbon::now()->subWeek();
            } elseif ($filter == 'lmonth') {
                $end_date = Carbon::now()->subMonth();
            } elseif ($filter == 'lyear') {
                $end_date = Carbon::now()->subYear();
            }
        }

        $criteria = collect();
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        return Order::getCOupons($criteria);
    }
}
