<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Customer;
use App\Models\Order;
use App\Models\Page;
use App\Models\Product;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;
use App\Exports\CustomerExport;
use Maatwebsite\Excel\Facades\Excel;
/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class CustomerController extends Controller
{
    protected $title = 'Customers';
    protected $viewPath = 'customers';
    protected $link = 'customers';

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    /**
     * Show the Dashboard.
     */
    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'orders-desc' => 'Sales Desc',
            'orders-asc' => 'Sales Asc',
            'name-desc' => 'Customer Name Desc',
            'name-asc' => 'Customer Name Asc',
            'city-desc' => 'Location Desc',
            'city-asc' => 'Location Asc',
            'order_price-desc' => 'Revenue Desc',
            'order_price-asc' => 'Revenue Asc',
            'users.last_login_at-desc' => 'Last Login Desc',
            'users.last_login_at-asc' => 'Last Login Asc',
        ];
        
            $page->last_logins = [
             'today' => 'Today',
             'week' => 'This Week',
             'lweek' => 'Last Week',
             'month' => 'This Month',
             'lmonth' => 'Last Month',
             'year' => 'This Year',
             'lyear' => 'Last Year',
             ];

        $filter = $request->filter;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';
        
        
        
        
        
        
        
          $count["total_customer"]=Customer::where('customer_type', 0)->count();

        $count["new_customer"]=Customer::where( 'created_at', '>', Carbon::now()->subMonth())
        ->where('customer_type', 0)
        ->select( DB::raw('count(distinct customers.id) as total'))->first()->total;
       
        $count["returning_customer"]=Customer::leftJoin("order_customers","order_customers.customer_id","customers.id")->
        leftJoin("orders","orders.id","order_customers.order_id") 
        ->where( 'orders.created_at', '>', Carbon::now()->subMonth(1))
        ->where('customer_type', 0)
        ->whereNotIn('customers.id', function ($query) {
            $query->select('order_customers.customer_id')->from('orders')->leftJoin("order_customers","orders.id","order_customers.order_id")
            ->where( 'orders.created_at', '>', Carbon::now()->subMonth(3))
            ->where( 'orders.created_at', '<', Carbon::now()->subMonth(1))->groupBy("order_customers.customer_id");
        })->select( DB::raw('count(distinct customers.id) as total'))->first()->total;

        $count["inactive_customer"]=Customer::whereNotIn('customers.id', function ($query) {
            $query->select('order_customers.customer_id')->from('orders')->leftJoin("order_customers","orders.id","order_customers.order_id")
            ->where( 'orders.created_at', '>', Carbon::now()->subMonth(3))->groupBy("order_customers.customer_id");
        })
        ->where('customer_type', 0)->select( DB::raw('count(distinct customers.id) as total'))->first()->total;


        $count["inactive_customer_percentage"]=round((($count["inactive_customer"]/ $count["total_customer"])*100),2);
        $count["guest_user"]=Customer::where( 'user_id', 0)
        ->select( DB::raw('count(distinct customers.id) as total'))->first()->total;

  $count["active_guest_user"]=Customer::where( 'user_id', 0) ->whereIn('customers.id', function ($query) {
    $query->select('order_customers.customer_id')->from('orders')
    ->leftJoin("order_customers","orders.id","order_customers.order_id")
    ->where( 'orders.created_at', '>', Carbon::now()->subMonth(3))
    ->groupBy("order_customers.customer_id");

})->select( DB::raw('count(distinct customers.id) as total'))->first()->total;
        
        if(isset($request->download)){
            
            
           $filter = $request->sort;
           $keyword = $request->keyword;
           
           $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
           $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
     
        
 
       

            return Excel::download(new CustomerExport($criteria), 'customers.xlsx');
        }
 
        return view($this->viewPath . '/lists', compact('page', 'request','count'));
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function lists(Request $request)
    {
        $sort = ['customers.id', 'desc'];
        
      //  DB::enableQueryLog();
        $detail = Customer::leftJoin('order_customers as oc', 'customers.id', 'oc.customer_id') 
        ->leftJoin('orders as o',function ($join) {
                $join->on('oc.order_id', '=' , 'o.id') ;
                $join->where('o.order_from', 1);
                $join->where('o.payment_status',  0);
            })->leftJoin('users', 'customers.user_id', 'users.id')
            ->select('customers.id','first_name', 'last_name','customers.email','customers.phone','o.billing_address1', 'o.billing_city', 'o.billing_postcode', 'last_login_at', DB::raw('count(o.id) as orders'), DB::raw('sum(o.paid_amount) as order_price'),'user_id');

        $detail->where('customer_type', 0);
        
        
        
        $detail->groupBy('customers.id');
 
            
      
        
        
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                  
                    $detail->where(function ($query) use ($names) {
                        $query->where(DB::raw("concat(first_name,' ',last_name)"), 'like', "%{$names}%")
                             ->orWhere('customers.email', 'like', "%{$names}%")
                            ->orWhere('customers.email', 'like', "%{$names}%")
                            ->orWhere('customers.phone', 'like', "%{$names}%");
                          //  ->orWhere('o.billing_city', 'like', "%{$names}%")
                            //->orWhere('o.billing_postcode', 'like', "%{$names}%");
                    });
                }

                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }





                $last_login_filter ='';
          
                if ($search['value'] != NULL && $search['name']=='last_login_filter')
                {
                    $last_login_filter =$search['value'];
             
                
              
  
              switch ($last_login_filter) {
                  case 'today':
                      $last_login_start_date = '';
                      $last_login_end_date = Carbon::now();
                      break;
                  case 'week':
                      $last_login_start_date = Carbon::now()->startOfWeek();
                      $last_login_end_date = Carbon::now();
                      break;
                  case 'month':
                      $last_login_start_date = Carbon::now()->startOfMonth();
                      $last_login_end_date = Carbon::now();
                      break;
                  case 'year':
                      $last_login_start_date = Carbon::now()->startOfYear();
                      $last_login_end_date = Carbon::now();
                      break;
                  case 'lweek':
                      $last_login_start_date = Carbon::now()->subDays(7);
                      $last_login_end_date = Carbon::now();
                      break;
                  case 'lmonth':
                      $last_login_start_date = Carbon::now()->subDays(30);
                      $last_login_end_date = Carbon::now();
                      break;
                  case "lyear":
                      $last_login_start_date = Carbon::now()->subDays(365);
                      $last_login_end_date = Carbon::now();
                      break;
                  default:
                      $last_login_start_date = '';
                      $last_login_end_date = '';
              }
  
              $last_login_start_date = $last_login_filter == '' ? $request->last_login_start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->last_login_start_date) : '' : $last_login_start_date;
              $last_login_end_date = $last_login_filter == '' ? $request->last_login_end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->last_login_end_date) : '' : $last_login_end_date;
             
  
              if ($last_login_start_date != NULL && $last_login_end_date != NULL) {
                  $detail->whereDate('users.last_login_at', '>=', $last_login_start_date)
                      ->whereDate('users.last_login_at', '<=', $last_login_end_date);
              } elseif ($last_login_start_date != NULL && $last_login_end_date == NULL) {
                  $detail->whereDate('users.last_login_at', '=', $last_login_start_date);
              } elseif ($last_login_start_date == NULL && $last_login_end_date != NULL) {
                  $detail->whereDate('users.last_login_at', '=', $last_login_end_date);
              }
                          
           
                }

              
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
            
            
            
            
            
            
        }
        
       
        if($request->order && $request->order[0]["column"]==7){
            $sort[0]="order_price";
            $sort[1]=$request->order[0]["dir"];
        }
        
           if($request->order && $request->order[0]["column"]==6){
            $sort[0]="orders";
            $sort[1]=$request->order[0]["dir"];
           }
  $detail->orderBy($sort[0], $sort[1]);
  //$detail->get();
 // dd(DB::getQueryLog());
        return DataTables::of($detail)
            ->editColumn('billing_postcode', function ($detail) {
                
                return $detail->billing_address1 . ', ' .$detail->billing_city . ', ' . strtoupper(str_replace(' ', '', $detail->billing_postcode));
            })
            ->editColumn('first_name', function ($detail) {
                return $detail->first_name . ' ' . $detail->last_name;
            })
            ->editColumn('order_price', function ($detail) {
                return '&pound; ' . round($detail->order_price, 2);
            })
             ->editColumn('last_login_at', function ($detail) {
    if($detail->last_login_at){
                 $to_date =  Carbon::parse($detail->last_login_at)->format('d/m/Y h:i a');
                 
                }else{

                    $to_date ="Long time ago";

                    if($detail->user_id==0){

                        $to_date ="Guest user";

                    }
                }
                 
                return $to_date;
             })
            ->addColumn('action', function ($detail) {
                $action = '';
                $sale_url = url('customers/sales/' . $detail->id);
                $action .= HtmlHelper::showButton($sale_url, 'Sale Analysis', 'shopping-cart');
                $product_url = url('customers/products/' . $detail->id);
                $action .= HtmlHelper::showButton($product_url, 'Product Analysis', 'cube');
                return $action;
            })
            ->removeColumn( 'last_name', 'billing_city','user','user_id','billing_address1')
            ->escapeColumns([])
            ->make(false);
    }

    public function customerIndex(Request $request, $id)
    {
        $customer = Customer::findOrFail($id);

        $page = collect();
        $page->title = 'Customer - ' . $customer->first_name . ' ' . $customer->last_name;
        $page->link = url('customers/sales/' . $id);
        $page->sorts = [
            'orders.created_at-desc' => 'Latest Order',
            'orders.created_at-asc' => 'Oldest Order',
            'orders.id-desc' => 'Order ID Desc',
            'orders.id-asc' => 'Order ID Asc',
            'quantity-desc' => 'Quantity Desc',
            'quantity-asc' => 'Quantity Asc',
            'paid_amount-desc' => 'Revenue Desc',
            'paid_amount-asc' => 'Revenue Asc',
        ];


        $filter = $request->filter;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/customer', compact('page', 'request', 'customer'));
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function customerLists(Request $request, $id)
    {
        $sort = ['orders.created_at', 'desc'];
        $detail = Order::select('orders.id', 'orders.created_at', 'd.quantity', 'orders.paid_amount')
            ->join('order_details as d', 'd.order_id', 'orders.id')
            ->join('order_customers as oc', 'orders.id', 'oc.order_id')
            ->online()
            ->where('oc.customer_id', $id)
            ->where('orders.payment_status', 0);

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('orders.id', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('orders.created_at', '>=', $from_date)
                    ->whereDate('orders.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('orders.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('orders.created_at', '=', $to_date);
            }
        }
        $detail->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })
            ->editColumn('paid_amount', function ($detail) {
                return '&pound; ' . round($detail->paid_amount, 2);
            })
            ->addColumn('action', function ($detail) {
                $sale_url = url('sales/' . $detail->id);
                return HtmlHelper::showButton($sale_url, 'Order View', 'shopping-cart');
            })
            ->removeColumn()
            ->escapeColumns([])
            ->make(false);
    }

    public function productIndex(Request $request, $id)
    {
        $customer = Customer::findOrFail($id);
        $page = collect();
        $page->title = 'Customer - ' . $customer->first_name . ' ' . $customer->last_name;
        $page->link = url('customers/products/' . $id);
        $page->sorts = [
            'quantity-desc' => 'Quantity Desc',
            'quantity-asc' => 'Quantity Asc',
            'products.title-desc' => 'Product Name Desc',
            'products.title-asc' => 'Product Name Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->id = $customer->id ?? '';
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';
        return view($this->viewPath . '/products', compact('page', 'request'));
    }

    public function productLists(Request $request, $id)
    {
        $sort = ['quantity', 'desc'];
        $detail = Product::select('products.title', DB::raw('sum(d.quantity) as quantity'))
            ->join('order_details as d', 'products.id', 'd.product_id')
            ->join('orders as o', 'd.order_id', 'o.id')
            ->join('order_customers as oc', 'o.id', 'oc.order_id')
            ->where('o.order_from', 1)
            ->where('oc.customer_id', $id);

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('products.title', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
        }
        $detail->groupBy('d.product_id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->removeColumn()
            ->escapeColumns([])
            ->make(false);
    }

    public function salesDataDashboard(Request $request)
    {
        $filter = $request->filter;
        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $details = Order::join('order_customers as oc', 'orders.id', 'oc.id')
            ->join('customers as c', 'oc.customer_id', 'c.id')
            ->when($filter != '' && $filter != 'all', function ($query) use ($filter, $start_date, $end_date) {
                $query->when($filter == 'today', function ($q) use ($end_date) {
                    $q->whereDate('orders.created_at', '=', $end_date);
                    return $q;
                })
                    ->when($filter != 'today', function ($q) use ($start_date, $end_date) {
                        $q->whereDate('orders.created_at', '>=', $start_date)
                            ->whereDate('orders.created_at', '<=', $end_date);
                        return $q;
                    });
            });

        $orders = $details->select('c.first_name', 'c.last_name', DB::raw('count(orders.id) as counts'))
            ->groupBy('c.id')
            ->orderBy('counts', 'desc')
            ->take(5)
            ->get();
        $prices = $details->select('c.first_name', 'c.last_name', DB::raw('count(orders.id) as counts'), DB::raw('sum(orders.paid_amount) as amount'))
            ->groupBy('c.id')
            ->orderBy('amount', 'desc')
            ->take(5)
            ->get();

        return compact('orders', 'prices');
    }

}
