<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Customer;
use App\Models\Doctor;
use App\Models\Order;
use App\Models\Prescription;
use App\Models\Product;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class DoctorController extends Controller
{
    protected $title = 'Doctors';
    protected $viewPath = 'doctors';
    protected $link = 'doctors';
    protected $orderStatus = ['0' => 'Pending', '1' => 'Active'];

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'created_at-desc' => 'Latest Joined',
            'created_at-asc' => 'Oldest Joined',
            'user_id-desc' => 'No Desc',
            'user_id-asc' => 'No Asc',
            'first_name-desc' => 'Doctor Name Desc',
            'first_name-asc' => 'Doctor Name Asc',
            'profession-desc' => 'Profession Desc',
            'profession-asc' => 'Profession Asc',
        ];


        $page->order_status = $this->orderStatus;
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/lists', compact('page', 'request'));
    }




    public function prescriberIndex(Request $request)
    {
        $page = collect();
        $page->title = 'Prescribers';
        $page->link = url('prescribers');

 
        $page->sorts = [
            'created_at-desc' => 'Latest Joined',
            'created_at-asc' => 'Oldest Joined',
            'user_id-desc' => 'No Desc',
            'user_id-asc' => 'No Asc',
            'first_name-desc' => 'Doctor Name Desc',
            'first_name-asc' => 'Doctor Name Asc',
            'profession-desc' => 'Profession Desc',
            'profession-asc' => 'Profession Asc',
        ];


        $page->order_status = $this->orderStatus;
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/lists', compact('page', 'request'));
    }
    public function lists(Request $request)
    {
        $sort = ['doctors.created_at', 'desc'];
        $detail =   Doctor::leftJoin('prescriptions as p', 'p.doctor_id', 'doctors.id')
            ->leftJoin('order_prescription as op', 'p.id', 'op.prescription_id')
            ->leftJoin('orders as o', 'op.order_id', 'o.id')->select(
                'doctors.id',
                'doctors.user_id',
                'doctors.first_name',
                'doctors.last_name',
                'doctors.email',
                'doctors.phone',
                'doctors.profession',
                'doctors.company',
                \DB::raw('count(o.id) as sales'),
                \DB::raw('sum(o.paid_amount) as revenue'),
                'doctors.address',
                'doctors.surgery_phone_no',
                'doctors.surgery_address',
                'doctors.created_at',
                'doctors.status'
            );


            $detail->where("is_prescriber",0);


        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;

            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('doctors.first_name', 'like', "%{$names}%")
                            ->orWhere('doctors.last_name', 'like', "%{$names}%")
                            ->orWhere('doctors.email', 'like', "%{$names}%")
                            ->orWhere('doctors.phone', 'like', "%{$names}%")
                            ->orWhere('doctors.profession', 'like', "%{$names}%")
                            ->orWhere('doctors.company', 'like', "%{$names}%")
                            ->orWhere('doctors.address', 'like', "%{$names}%")
                            ->orWhere('doctors.surgery_phone_no', 'like', "%{$names}%")
                            ->orWhere('doctors.user_id', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'status') {
                    $detail->where('status', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }

            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('doctors.created_at', '>=', $from_date)
                    ->whereDate('doctors.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('doctors.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('doctors.created_at', '=', $to_date);
            }
        }

        $detail->groupBy('doctors.id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('user_id', function ($detail) {
                return  $detail->user_id;
            })->editColumn('first_name', function ($detail) {
                return $detail->first_name . ' ' . $detail->last_name;
            })
            ->editColumn('email', function ($detail) {
                return   $detail->email;
            })
            ->editColumn('phone', function ($detail) {
                return   $detail->phone;
            })
            ->editColumn('company', function ($detail) {
                return  $detail->company;
            })->editColumn('profession', function ($detail) {
                return  $detail->profession;
            })
            ->editColumn('sales', function ($detail) {
                return $detail->sales;
            })->editColumn('revenue', function ($detail) {

                return '&pound; ' . round($detail->revenue, 2);
            })
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })
            ->editColumn('status', function ($detail) {
                return $this->orderStatus[$detail->status];
            })->addColumn('action', function ($detail) {
                $action = '';
                $sale_url = url('doctors/customers/' . $detail->id);
                $action .= HtmlHelper::showButton($sale_url, 'Doctor Patients', 'users');
                $product_url = url('doctors/products/' . $detail->id);
                $action .= HtmlHelper::showButton($product_url, 'Product Analysis', 'cube');
                return $action;
            })
            ->removeColumn(
                'id',
                'last_name',
                'address',
                'surgery_phone_no',
                'surgery_address'
            )
            ->escapeColumns([])
            ->make(false);
    }



    public function prescriberLists(Request $request)
    {
        $sort = ['doctors.created_at', 'desc'];
        $detail =   Doctor::leftJoin('prescriptions as p', 'p.doctor_id', 'doctors.id')
            ->leftJoin('order_prescription as op', 'p.id', 'op.prescription_id')
            ->leftJoin('orders as o', 'op.order_id', 'o.id')->select(
                'doctors.id',
                'doctors.user_id',
                'doctors.first_name',
                'doctors.last_name',
                'doctors.email',
                'doctors.phone',
                'doctors.profession',
                'doctors.company',
                \DB::raw('count(o.id) as sales'),
                \DB::raw('sum(o.paid_amount) as revenue'),
                'doctors.address',
                'doctors.surgery_phone_no',
                'doctors.surgery_address',
                'doctors.created_at',
                'doctors.status'
            );

            $detail->where("is_prescriber",1);


        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;

            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('doctors.first_name', 'like', "%{$names}%")
                            ->orWhere('doctors.last_name', 'like', "%{$names}%")
                            ->orWhere('doctors.email', 'like', "%{$names}%")
                            ->orWhere('doctors.phone', 'like', "%{$names}%")
                            ->orWhere('doctors.profession', 'like', "%{$names}%")
                            ->orWhere('doctors.company', 'like', "%{$names}%")
                            ->orWhere('doctors.address', 'like', "%{$names}%")
                            ->orWhere('doctors.surgery_phone_no', 'like', "%{$names}%")
                            ->orWhere('doctors.user_id', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'status') {
                    $detail->where('status', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }

            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('doctors.created_at', '>=', $from_date)
                    ->whereDate('doctors.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('doctors.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('doctors.created_at', '=', $to_date);
            }
        }

        $detail->groupBy('doctors.id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('user_id', function ($detail) {
                return  $detail->user_id;
            })->editColumn('first_name', function ($detail) {
                return $detail->first_name . ' ' . $detail->last_name;
            })
            ->editColumn('email', function ($detail) {
                return   $detail->email;
            })
            ->editColumn('phone', function ($detail) {
                return   $detail->phone;
            })
            ->editColumn('company', function ($detail) {
                return  $detail->company;
            })->editColumn('profession', function ($detail) {
                return  $detail->profession;
            })
            ->editColumn('sales', function ($detail) {
                return $detail->sales;
            })->editColumn('revenue', function ($detail) {

                return '&pound; ' . round($detail->revenue, 2);
            })
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })
            ->editColumn('status', function ($detail) {
                return $this->orderStatus[$detail->status];
            })->addColumn('action', function ($detail) {
                $action = '';
                $sale_url = url('doctors/customers/' . $detail->id);
                $action .= HtmlHelper::showButton($sale_url, 'Doctor Patients', 'users');
                $product_url = url('doctors/products/' . $detail->id);
                $action .= HtmlHelper::showButton($product_url, 'Product Analysis', 'cube');
                return $action;
            })
            ->removeColumn(
                'id',
                'last_name',
                'address',
                'surgery_phone_no',
                'surgery_address'
            )
            ->escapeColumns([])
            ->make(false);
    }
    public function productIndex(Request $request, $id)
    {
        $customer = Doctor::findOrFail($id);
        $page = collect();
        $page->title = 'Doctor - ' . $customer->first_name . ' ' . $customer->last_name . " Products List";
        $page->link = url('doctors/products/' . $id);
        $page->sorts = [
            'quantity-desc' => 'Quantity Desc',
            'quantity-asc' => 'Quantity Asc',
            'products.title-desc' => 'Product Name Desc',
            'products.title-asc' => 'Product Name Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->id = $customer->id ?? '';
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';
        return view($this->viewPath . '/products', compact('page', 'request'));
    }


    public function productLists(Request $request, $id)
    {
        $sort = ['quantity', 'desc'];
        $detail = Product::select('products.title', \DB::raw('sum(d.quantity) as quantity'))
            ->join('order_details as d', 'products.id', 'd.product_id')
            ->join('orders as o', 'd.order_id', 'o.id')
            ->join('order_prescription as op', 'o.id', 'op.order_id')
            ->join('prescriptions as p', 'p.id', 'op.prescription_id')

            ->where('o.order_from', 1)
            ->where('p.doctor_id', $id);



        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('products.title', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
        }
        $detail->groupBy('d.product_id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->removeColumn()
            ->escapeColumns([])
            ->make(false);
    }



    public function customersLists(Request $request, $id)
    {
        $sort = ['orders', 'desc'];




        $sort = ['prescriptions.created_at', 'desc'];
        $detail = Prescription::join('order_prescription as op', 'prescriptions.id', 'op.prescription_id')
            ->join('orders as o', 'op.order_id', 'o.id')
            ->join('order_customers as oc', 'oc.order_id', 'o.id')
            ->join('prescription_medicines as pm', 'pm.prescription_id', 'prescriptions.id')

            ->where('o.order_from', 1)
            ->select('prescriptions.first_name', 'prescriptions.last_name', 'o.billing_city', 'o.billing_postcode', DB::raw('group_concat(pm.title) as medicine'), 'o.paid_amount as amount', 'oc.customer_id');
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('prescriptions.first_name', 'like', "%{$names}%")
                            ->orWhere('prescriptions.last_name', 'like', "%{$names}%")
                            ->orWhere('billing_city', 'like', "%{$names}%")
                            ->orWhere('billing_postcode', 'like', "%{$names}%");
                    });
                }

                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
        }


        $detail->groupBy('prescriptions.id');
        $detail->orderBy($sort[0], $sort[1]);


        return DataTables::of($detail)->editColumn('first_name', function ($detail) {
            return $detail->first_name . ' ' . $detail->last_name;
        })->editColumn('billing_postcode', function ($detail) {
            return $detail->billing_city . ', ' . strtoupper(str_replace(' ', '', $detail->billing_postcode));
        })
            ->editColumn('medicine', function ($detail) {
                return  $detail->medicine;
            })
            ->editColumn('amount', function ($detail) {
                return '&pound; ' . round($detail->amount, 2);
            })
            ->addColumn('action', function ($detail) {

                $action = '';
                $sale_url = url('customers/sales/' . $detail->customer_id);
                $action .= HtmlHelper::showButton($sale_url, 'Sale Analysis', 'shopping-cart');


                return $action;
            })
            ->removeColumn('id', 'last_name', 'billing_city', 'customer_id')
            ->escapeColumns([])
            ->make(false);
    }


    public function customersIndex(Request $request, $id)
    {

        $customer = Doctor::findOrFail($id);
        $page = collect();
        $page->title = 'Doctor - ' . $customer->first_name . ' ' . $customer->last_name . " Prescriptions Lists";

        $page->link = url($this->link . "/customers/" . $id);
        $page->sorts = [
            'orders-desc' => 'Sales Desc',
            'orders-asc' => 'Sales Asc',
            'name-desc' => 'Customer Name Desc',
            'name-asc' => 'Customer Name Asc',
            'city-desc' => 'Location Desc',
            'city-asc' => 'Location Asc',
            'order_price-desc' => 'Revenue Desc',
            'order_price-asc' => 'Revenue Asc',
        ];

        $filter = $request->filter;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/customer', compact('page', 'request'));
    }
}
