<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Doctor;
use App\Models\HcpOrder;
use App\Models\Prescription;
use App\Models\Order;
use App\Models\Staff;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class HcpSaleController extends Controller
{
    protected $title = 'H C P Sales';
    protected $viewPath = 'sales/hcp';
    protected $link = 'sales/hcp';
    protected $orderStatus = ['0' => 'Pending Quotes', '1' => 'Awaiting Payment', '2' => 'Processing', '3' => 'Completed'];

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->order_status = $this->orderStatus;

        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
            'invoice_no-desc' => 'Invoice No Desc',
            'invoice_no-asc' => 'Invoice No Asc',
            'billing_first_name-desc' => 'Customer Name Desc',
            'billing_first_name-asc' => 'Customer Name Asc',
            'billing_city-desc' => 'Location Desc',
            'billing_city-asc' => 'Location Asc',
            'paid_amount-desc' => 'Revenue Desc',
            'paid_amount-asc' => 'Revenue Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath, compact('page', 'request'));
    }

    public function lists(Request $request)
    {
    
        $sort = ['created_at', 'desc'];
     
        $detail = Order::select('orders.id', 'orders.order_no', 'orders.invoice_no', 'orders.created_at', 'orders.billing_first_name', 'orders.billing_last_name', 'orders.billing_city', 'orders.billing_postcode', 'orders.paid_amount', 'orders.order_status')
        ->leftJoin('order_prescription  as op', 'orders.id', 'op.order_id')
        ->rightJoin('prescriptions  as p', 'p.id', 'op.prescription_id')
        ->paid()->online();
 

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('billing_first_name', 'like', "%{$names}%")
                            ->orWhere('billing_last_name', 'like', "%{$names}%")
                            ->orWhere('billing_city', 'like', "%{$names}%")
                            ->orWhere('billing_postcode', 'like', "%{$names}%")
                            ->orWhere('invoice_no', 'like', "%{$names}%")
                            ->orWhere('order_no', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'status') {
                    $detail->where('p.status', $search['value']);
 
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('orders.created_at', '>=', $from_date)
                    ->whereDate('orders.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('orders.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('orders.created_at', '=', $to_date);
            }
        }
 

        $detail->groupBy('id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('billing_first_name', function ($detail) {
                return $detail->billing_first_name . ' ' . $detail->billing_last_name;
            })
            ->editColumn('billing_postcode', function ($detail) {
                return $detail->billing_city . ', ' . strtoupper(str_replace(' ', '', $detail->billing_postcode));
            })
            ->editColumn('paid_amount', function ($detail) {
                return '&pound; ' . round($detail->paid_amount, 2);
            })
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })    ->editColumn('order_status', function ($detail) {
                return $this->orderStatus[$detail->order_status];
            })
            ->addColumn('action', function ($detail) {
                $product_url = url('sales/' . $detail->id);
                return HtmlHelper::showButton($product_url, 'Order Detail', 'link');
            })
            ->removeColumn('id', 'billing_last_name', 'billing_city')
            ->escapeColumns([])
            ->make(false);
    }



    public function show($id)
    {
        $order = Order::findOrFail($id);
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->order_status = $this->orderStatus;
        return view($this->viewPath . '/detail', compact('page', 'order'));
    }

     public function salesGraph(Request $request)
    {
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? 'all';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
       
        return HcpOrder::getBySaleDay($criteria);
    }

    public function revenueGraph(Request $request)
    {
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        return HcpOrder::getByRevenueDay($criteria);
    }

    public function salesData(Request $request)
    {
        
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;
        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        
        $order = number_format(HcpOrder::getAll($criteria) ->leftJoin('order_details as d', 'orders.id', 'd.order_id')
        ->where('d.pres_status', 1)->active()->count());
        
        $revenue = number_format(HcpOrder::getAll($criteria) ->leftJoin('order_details as d', 'orders.id', 'd.order_id')
        ->where('d.pres_status', 1)->active()->sum('paid_amount'), 2);

       
        return compact('order', 'revenue');
    } 





    public function prescribeNow(Request $request)
    {
        $page = collect();
 

        $page->title = "Prescribe Now";
        $page->link = url('prescribe-now');
        $page->order_status = $this->orderStatus;

        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
            'invoice_no-desc' => 'Invoice No Desc',
            'invoice_no-asc' => 'Invoice No Asc',
            'billing_first_name-desc' => 'Customer Name Desc',
            'billing_first_name-asc' => 'Customer Name Asc',
            'billing_city-desc' => 'Location Desc',
            'billing_city-asc' => 'Location Asc',
            'paid_amount-desc' => 'Revenue Desc',
            'paid_amount-asc' => 'Revenue Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;

        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

    
        

        return view('sales/prescibe_now', compact('page', 'request'));
    }



    public function prescribeSalesData(Request $request)
    {
        
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;
        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        
        $order = number_format(HcpOrder::getAll($criteria) ->leftJoin('order_details as d', 'orders.id', 'd.order_id')
        ->where('d.pres_status', 1)->active()->count());
        
        $revenue = number_format(HcpOrder::getAll($criteria) ->leftJoin('order_details as d', 'orders.id', 'd.order_id')
        ->where('d.pres_status', 1)->active()->sum('paid_amount'), 2);

        $doctor =number_format(Doctor::where("is_prescriber",0)->get()->count());
        $prescribers =number_format(Doctor::where("is_prescriber",1)->get()->count());
        
        $staff =number_format(Staff::all()->count());
        



        return compact('order', 'revenue','doctor','staff','prescribers');
    } 
}
