<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Content;
use App\Models\Product;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class LoginController extends Controller
{
    protected $title = 'Customers';
    protected $viewPath = 'customers';
    protected $link = 'customers';

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    /**
     * Show the Dashboard.
     */
    public function index(Request $request, $decrypt)
    {
        $ciphering = 'AES-128-CBC';
        $secret_key = 'SMANALYTICS';
        $secret_iv = date('d/m/Y');

        $key = substr(hash('sha256', $secret_key), 0, 16);
        $iv = substr(hash('sha256', $secret_iv), 0, 16);

        $decrypt_string = base64_decode($decrypt);
        $decrypt_value = openssl_decrypt($decrypt_string, $ciphering, $key, 0, $iv);
        if ($decrypt_value === false) {
            return redirect(url(env('BB_URL')));
        }
        $user = User::find($decrypt_value);
        Auth::login($user);
        if (Auth::user()) {
            return redirect('/');
        }
        return redirect(env('BB_URL'));
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function lists(Request $request)
    {
        $detail = DB::table('orders as o')
            ->select('c.id', 'c.title', DB::raw('sum(oc.qty) as quantity'))
            ->join('order_carts as oc', 'o.id', 'oc.order_id')
            ->join('products as p', 'p.id', 'oc.product_id')
            ->join('contents as c', 'p.content_id', 'c.id')
            ->where('o.payment', 1);

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('c.title', 'like', "%{$names}%");
                }

                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereBetween('o.datetime', [$from_date, $to_date]);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.datetime', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.datetime', '=', $to_date);
            }
        }
        $detail->groupBy('c.id', 'c.title')
            ->orderBy('quantity', 'desc');
        return DataTables::of($detail)
            ->addColumn('action', function ($detail) {
                $action = '';
                $edit_url = url('product/' . $detail->id . '/' . Str::slug($detail->title));
                $action .= HtmlHelper::viewButton($edit_url);
                return $action;
            })
            ->removeColumn('id')
            ->escapeColumns([])
            ->make(false);
    }

    public function productIndex(Request $request, $id, $slug)
    {
        $product = Content::findOrFail($id);
        if (Str::slug($product->title) == $slug) {
            $page = collect();
            $page->title = 'Product - ' . $product->title;
            $page->link = url('product/' . $id . '/' . $slug);

            $filter = $request->filter;
            $keyword = $request->keyword;

            switch ($filter) {
                case 'today':
                    $start_date = '';
                    $end_date = Carbon::now();
                    break;
                case 'week':
                    $start_date = Carbon::now()->startOfWeek();
                    $end_date = Carbon::now();
                    break;
                case 'month':
                    $start_date = Carbon::now()->startOfMonth();
                    $end_date = Carbon::now();
                    break;
                case 'year':
                    $start_date = Carbon::now()->startOfYear();
                    $end_date = Carbon::now();
                    break;
                case 'lweek':
                    $start_date = Carbon::now()->subDays(7);
                    $end_date = Carbon::now();
                    break;
                case 'lmonth':
                    $start_date = Carbon::now()->subDays(30);
                    $end_date = Carbon::now();
                    break;
                case "lyear":
                    $start_date = Carbon::now()->subDays(365);
                    $end_date = Carbon::now();
                    break;
                default:
                    $start_date = '';
                    $end_date = '';
            }

            $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
            $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

            $criteria = collect();
            $criteria->id = $product->id ?? '';
            $criteria->keyword = $keyword ?? '';
            $criteria->type = $filter ?? '';
            $criteria->start_date = $start_date;
            $criteria->end_date = $end_date;

            $products = Product::getByProductSales($criteria);
            $counts = collect();
            $counts->total_products = Product::getByProductSalesCount($criteria);
            $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
            $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

            return view('product', compact('counts', 'page', 'products', 'request'));
        }
        abort('404');
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function productLists(Request $request, $id, $slug)
    {
        $detail = DB::table('orders as o')
            ->select('o.id', 'o.datetime', 'u.name', 'u.lname', 'u.postcode', 'oc.qty')
            ->leftJoin('users as u', 'u.id', 'o.user_id')
            ->join('order_carts as oc', 'o.id', 'oc.order_id')
            ->join('products as p', 'p.id', 'oc.product_id')
            ->join('contents as c', 'p.content_id', 'c.id')
            ->where('o.payment', 1)
            ->where('c.id', $id);

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('u.name', 'like', "%{$names}%")
                            ->orWhere('o.id', 'like', "%{$names}%")
                            ->orWhere('u.lname', 'like', "%{$names}%")
                            ->orWhere('o.postcode', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereBetween('o.datetime', [$from_date, $to_date]);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.datetime', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.datetime', '=', $to_date);
            }
        }
        $detail->orderBy('o.datetime', 'desc');
        return DataTables::of($detail)
            ->editColumn('name', function ($detail) {
                return $detail->name . ' ' . $detail->lname;
            })
            ->editColumn('datetime', function ($detail) {
                return Carbon::parse($detail->datetime)->format('d/m/Y h:i a');
            })
            ->removeColumn('lname')
            ->escapeColumns([])
            ->make(false);
    }

}
