<?php

namespace App\Http\Controllers;

use App\Helpers\DeliveryHelper;
use App\Helpers\HtmlHelper;
use App\Helpers\OrderHelper;
use App\Models\Courier;
use App\Models\Doctor;
use App\Models\HcpOrder;
use App\Models\Prescription;
use App\Models\Order;
use App\Models\Patient;
use App\Models\Staff;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class OrderController extends Controller
{
    protected $title = 'Delivery';
    protected $viewPath = 'orders/lists';
    protected $link = 'delivery/postage';
    protected $orderStatus = ['0' => 'Same Day Delivery', '1' => 'Next Day Delivery', '2' => 'Normal Delivery','3' => 'Pending', '4' => 'Completed'];

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->order_status = $this->orderStatus;

        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
            'invoice_no-desc' => 'Invoice No Desc',
            'invoice_no-asc' => 'Invoice No Asc',
            'billing_first_name-desc' => 'Customer Name Desc',
            'billing_first_name-asc' => 'Customer Name Asc',
            'billing_city-desc' => 'Location Desc',
            'billing_city-asc' => 'Location Asc',
            'paid_amount-desc' => 'Revenue Desc',
            'paid_amount-asc' => 'Revenue Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
         
        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') :'';
          
        //\DB::connection()->enableQueryLog();
        $orders=Order::active()->with("shippingMethod")
        ->where('created_at',">=", $criteria->start_date==''?Carbon::createFromFormat('d/m/Y', "01/01/1700")->format('d/m/Y'):$criteria->start_date)
        ->where("created_at","<=", $criteria->end_date==''?Carbon::now():$criteria->end_date)
        ->select("shipping_method",  \DB::raw('count(shipping_method) as orders',"shipping_methods.title,shipping_methods.description,shipping_methods.amount"))
        ->groupBy("orders.shipping_method")->where("shipping_method","!=",0)
        ->orderBy("orders","DESC")->get(); 
        //$query=\DB::getQueryLog();        
        //var_dump($query);
        //die;
 

        return view($this->viewPath, compact('page', 'request','orders'));
    }
 

    public function salesData(Request $request)
    {
        
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;
        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $today = Carbon::now();

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '01/01/1700' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : $today : $end_date;
        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
    
        

        if($request->start_date==null&&$request->end_date==null){
            $next_days = number_format(Order::active()->whereIn('shipping_method', [2, 15])->count());
           
            $same_days = number_format(Order::active()->whereIn('shipping_method', [14, 16,17,18,19])->count());
            $normal_days = number_format(Order::active()->whereIn('shipping_method', [1, 13,3,4])->count());
            // $postage = number_format(Order::getAll($criteria)->active()->where('created_at', $today)->whereIn('shipping_method', [14, 16])->count());
            
            $orders=Order::with("shippingMethod")->select("shipping_method",  \DB::raw('count(shipping_method) as orders',"shipping_methods.title,shipping_methods.description,shipping_methods.amount"))->groupBy("orders.shipping_method")->where("shipping_method","!=",0)->orderBy("orders","DESC")->get();
            $postage=0;
            
            foreach($orders as $order){
                $postage=$postage+$order->shippingMethod->amount*$order->orders;
            }
 
        }else{


            $next_days = number_format(Order::active()
            ->where("created_at",">=", $criteria->start_date==''?Carbon::now()->format('d/m/Y'):$criteria->start_date->format('Y-m-d'))
            ->where("created_at","<=", $criteria->end_date==''?Carbon::now()->format('d/m/Y'):$criteria->end_date->format('Y-m-d'))
 
            ->whereIn('shipping_method', [2, 15])->count());
 
            $same_days = number_format(Order::active()
            ->where('created_at',">=", $criteria->start_date==''?Carbon::createFromFormat('d/m/Y', "01/01/1700")->format('d/m/Y'):$criteria->start_date->format('Y-m-d'))
            ->where("created_at","<=", $criteria->end_date==''?Carbon::now()->format('d/m/Y'):$criteria->end_date->format('Y-m-d'))
            ->whereIn('shipping_method', [14, 16,17,18,19])->count());
            $normal_days = number_format(Order::active()
            ->where('created_at',">=", $criteria->start_date==''?Carbon::createFromFormat('d/m/Y', "01/01/1700")->format('d/m/Y'):$criteria->start_date->format('Y-m-d'))
            ->where("created_at","<=", $criteria->end_date==''?Carbon::now()->format('d/m/Y'):$criteria->end_date->format('Y-m-d'))
            ->whereIn('shipping_method', [1, 13,3,4])->count());
 //           $postage =Order::with("shippingMethod")->select("shipping_method",  \DB::raw('count(shipping_method) as orders',"shipping_methods.title,shipping_methods.description,shipping_methods.amount*orders as total"))->groupBy("orders.shipping_method")->where("shipping_method","!=",0)->orderBy("orders","DESC")->get();
 
 $orders=Order::with("shippingMethod")->select("shipping_method",  \DB::raw('count(shipping_method) as orders',"shipping_methods.title,shipping_methods.description,shipping_methods.amount"))->groupBy("orders.shipping_method")
 ->where('created_at',">=", $criteria->start_date==''?Carbon::createFromFormat('d/m/Y', "01/01/1700")->format('d/m/Y'):$criteria->start_date->format('Y-m-d'))
 ->where("created_at","<=", $criteria->end_date==''?Carbon::now()->format('d/m/Y'):$criteria->end_date->format('Y-m-d'))

 ->where("shipping_method","!=",0)->orderBy("orders","DESC")->get();
 $postage=0;
            foreach($orders as $order){
                $postage=$postage+$order->shippingMethod->amount*$order->orders;
            }

//  var_dump($postage);

           
        }



        $postage = "£ ".$postage;
        return compact('next_days', 'same_days','normal_days','postage');
    }


     /**
     * Display a listing of the resource in datatable.
     * @return array
     * @throws \Exception
     */
    public function DeliveryLists(Request $request, $id = '')
    {

      

        $store = session('website_id');
        if ($store == '')
            $store = 1;

        $couriers = Courier::pluck('title', 'id');

        //   \DB::connection()->enableQueryLog();
        $detail = Order::active()->select([
            'orders.shipping_ref_no',
            'orders.id',
            'orders.order_no',
            'orders.created_at', 
            'orders.order_no as customer_name',
            'orders.order_no as productname',
            \DB::raw('SUM(d.quantity) as product_count'),
            'orders.order_status',
            \DB::raw('CONCAT(shipping_first_name," ",shipping_last_name) as ship_name'),
            'm.title as shipping_method_title',
            'orders.paid_amount as customer_email',
            'orders.payment_status',
            'orders.paid_amount',
            'cr.title as courier_company',
            'orders.invoice_no',
            'shipping_city',
            'shipping_postcode',
            'orders.flag',
            'orders.order_from',
            'orders.order_type',
            'orders.pharmacist',
            'orders.vat',
            'orders.tell_gp',
            'orders.contacted_gp',
            'd.consultation_type',
           
        ])
            ->leftJoin('order_details as d', 'orders.id', 'd.order_id')
            ->leftJoin('shipping_methods as m', 'm.id', '=', 'orders.shipping_method')
            ->leftJoin('couriers as cr', 'cr.id', 'orders.courier_id');
           
        if ($id != '') {
            $detail->join('order_customers as c', 'orders.id', 'c.order_id')
                ->where('c.customer_id', $id);
        }
         
    

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                   

                    $detail->where(function ($query) use ($names) {
                        $query->where('orders.order_no', $names)
                            ->orWhere('orders.shipping_ref_no', 'like', "%{$names}%")
                             ->orWhere('orders.id', $names)
                            ->orWhere('orders.shipping_postcode', $names)
                            ->orWhere('orders.billing_postcode', $names)
                            // ->orWhere('orders.paid_amount', $names)
                            // ->orWhere('m.title', 'like', "%{$names}%")
                            // ->orWhere('d.title', 'like', "%{$names}%")
                            ->orWhereHas('customer', function ($query1) use ($names) {
                                $query1->whereRaw("concat(first_name, ' ', last_name) like '%{$names}%' ")
                                    ->orWhere('customers.id', $names)
                                    ->orWhere('customers.email', 'like', "%{$names}%");
                            });
                    });


                   
     
       
                }
                if ($search['value'] != NULL && $search['name'] == 'status') {


                    if($search['value']==0){
                        //same day
                          $detail->whereIn('shipping_method', [14,16, 17,18,19]);

                    }else if($search['value']==1){
                        //next day
                        $detail->whereIn('shipping_method', [2,15]);

                     }else if($search['value']==2){
                        //normal delivery
                        $detail->whereIn('shipping_method', [1,13,3,4]);

                     }else if($search['value']==3){
                        //Pending
                        $detail->where('order_status', 0);


                     }else if($search['value']==2){
                        //completed
                        $detail->where('order_status', 4);


                     }

                  
 
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
               

                $detail->whereDate('orders.created_at', '>=', $from_date->format('Y-m-d'))
                    ->whereDate('orders.created_at', '<=', $to_date->format('Y-m-d'));
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('orders.created_at', '=', $from_date->format('Y-m-d'));
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('orders.created_at', '=', $to_date->format('Y-m-d'));
            }
        }







          $detail
        //   ->where('orders.order_type','<>', 4)
          ->groupBy('orders.id');
        //  var_dump($detail->count());
        //  
         return DataTables::of($detail)->editColumn('shipping_ref_no', function ($detail) { 

            return $detail->shipping_ref_no;
        })
            ->editColumn('id', function ($detail) { //1
                $order = '<a href="sales/' . $detail->id . '">#' . $detail->order_no . '</a>';
                if ($detail->flag == 1 || $detail->flag == 2) {
                    $order = '<i class="fa fa-flag" aria-hidden="true"></i> <br>' . $order;
                }
                $adminOrder = Order::find($detail->id);
                if ($adminOrder!=null and $adminOrder->admin_order_status == 1) {
                    $order = '<i class="fa fa-user" aria-hidden="true"></i> - ' . $order;
                }
                return $order;
            })
            ->editColumn('customer_name', function ($detail) { //a1
                $customer_data = "";
                $customer = $detail->customer->first();
                if ($customer) {
                    $customer_data = '<a href="customers/' . $customer->id . '/edit" target="_blank">';
                    if ($customer->verification_status == 0) {
                        $customer_data = $customer_data . $customer->first_name . ' ' . $customer->last_name;
                    } else {
                        $customer_data = $customer_data . "<span style='color:#2aba77;'><b>" . $customer->first_name . ' ' . $customer->last_name . "</b></span>";
                    }
                    if ($customer->flag == 1) {
                        $customer_data = '<i class="fa fa-flag" aria-hidden="true"></i> - ' . $customer_data;
                    }
                    $customer_data = $customer_data . '</a>';
                    if (OrderHelper::is_bmi_order($detail->id)) {
                        $bmis = $customer->latest_bmis;
                        if ($bmis!=null &&$bmis->count() > 0) {
                            $i = 1;
                            foreach ($bmis as $bmi) {
                                if ($bmi->bmi_index > 28) {
                                    $index = "<span style='color:#2aba77;'>" . $bmi->bmi_index . "</span>";
                                } else {
                                    $index = "<span style='color:red;'>" . $bmi->bmi_index . "</span>";
                                }
                                $customer_data = $customer_data . '<br>' . $i . '. ' . $index . "(" . Carbon::parse($bmi->created_at)->format('d M Y') . ")";
                                $i++;
                            }
                        }
                    }
                    if ($detail->tell_gp == 1 && $detail->contacted_gp == 0) {
                        $customer_data = $customer_data . '<br><span style="color:red;"> Contact GP</span>';
                    }
                    if ($detail->tell_gp == 1 && $detail->contacted_gp == 1) {
                        $customer_data = $customer_data . '<br><span style="color:green;"> GP Contacted</span>';
                    }
                    if($customer->notes)
                      {
                          $notes_count = $customer->notes->count();
                      }else{
                        $notes_count =0;
                      }

                    if ($notes_count > 0) {
                        $customer_data = $customer_data . '<br><a href="javascript:void(0);" onclick="listNote(' . $customer->id . ', ' . $detail->id . ')" ><i class="fa fa-sticky-note" aria-hidden="true"></i></a>';
                    }
                }
                return $customer_data;
            })
            ->addColumn('customer_email', function ($detail) { //a2
                $email = "";
                if($detail->customer){
                $customer = $detail->customer->first();
                if ($customer)
                    $email = $customer->email;
                }
                return $email;
            })
            ->editColumn('productname', function ($detail) { //a3
                $allproducts = '';
                foreach ((array) $detail->items as $item) {
                    if ($allproducts != '') {
                        $allproducts = $allproducts . ' , ' . $item->title;
                    } else {
                        $allproducts = $item->title;
                    }
                }
                return $allproducts;
            })
            ->editColumn('ship_name', function ($detail) { //e2
                $shipping_address = $detail->ship_name . "<br>" . $detail->shipping_city;
                $sameday_postcode = DeliveryHelper::is_sameday_postcode($detail->shipping_postcode);
                $shipping_postcode = $detail->shipping_postcode;
                if ($sameday_postcode) {
                    $shipping_postcode = "<span style='background-color:#f5a966; color:#fff; padding: 0px 10px 2px;'><b>" . $detail->shipping_postcode . "</b></span>";
                }
                $shipping_address .= "<br>" . $shipping_postcode;
                return $shipping_address;
            })
            ->editColumn('shipping_method_title', function ($detail) { //e3
                $delivery_day = DeliveryHelper::get_order_delivery_day($detail->id);
                $shipping_method = $detail->shipping_method_title;
                if ($delivery_day)
                    $shipping_method = str_replace('NEXTDAY', $delivery_day, $shipping_method);

                if ($shipping_method == "Same Day Delivery") {
                    $shipping_method = "<span style='color:#2aba77;'><b>" . $shipping_method . "</b></span>";
                }
                return $shipping_method;
            })
            ->editColumn('shipping_ref_no', function ($detail) use ($couriers) { //e4
                $shipping_info = "";
                if ($detail->shipping_ref_no) {
                    $shipping_info .= $detail->shipping_ref_no;
                } elseif ($detail->order_status == 1 || $detail->order_status == 2 || $detail->order_status == 3) {
                    $shipping_info .= '<input type="text" data-id="' . $detail->id . '" data-name="ref_no" class="form-control ref_input" id="' . $detail->id . '_ref_no">';
                }
                if ($detail->courier_company) {
                    $shipping_info .= ": " . $detail->courier_company;
                } elseif ($detail->order_status == 1 || $detail->order_status == 2 || $detail->order_status == 3) {
                    $shipping_info .= "<br>" . \Form::select('courier_id', $couriers, '1', ['class' => '', 'data-id' => $detail->id, 'id' => 'courier_id']);
                }
                return $shipping_info;
            })
            ->editColumn('created_at', function ($detail) { //e5
                return Carbon::parse($detail->created_at)->format('d M Y');
            })
            ->editColumn('paid_amount', function ($detail) { //e6
                return '£ ' . $detail->paid_amount;
            })
            ->editColumn('payment_status', function ($detail) { //e7
                if ($detail->payment_status == 0) {
                    $status = 'Paid';
                } elseif ($detail->payment_status == 2) {
                    $status = 'Processing';
                } else {
                    $status = 'Unpaid';
                }
                return $status;
            })
            ->editColumn('order_status', function ($detail) use ($request) { //e8
                $status = '';
                if ($detail->order_status == 0) {
                    $status = 'Pending';
                } elseif ($detail->order_status == 1 || $detail->order_status == 2 || $detail->order_status == 3 || $detail->order_status == 6 || $detail->order_status == 7) {
                    $status = 'Processing';
                } elseif ($detail->order_status == 4) {
                    $status = 'Completed';
                } elseif ($detail->order_status == 5 || $detail->order_status == 8) {
                    $status = 'Cancelled';
                } elseif ($detail->order_status == 9) {
                    $status = 'On Hold';
                } else {
                    $status = '';
                }
                if ($detail->consultation_type == 0) {
                    $status = 'Query Orders';
                }
                if ($detail->consultation_type == 0) {
                    $status = 'Query Orders';
                }
                return $status;
            })
            ->addColumn('action', function ($detail) { //a4
               
 
                $sale_url ='';
                $action = '';
                if($detail->customer){
                    $customer = $detail->customer->first();
                    if ($customer){
                $sale_url = url('doctors/customers/' . $customer->id);
                    }

                    $action .= HtmlHelper::showButton($sale_url, 'Customer', 'users');
                }
              
                $product_url = url('doctors/products/' . $detail->id);
                $action .= HtmlHelper::showButton($product_url, 'Prescriptions', 'cube');
                return $action;
            })
            ->addColumn('checkrows', function ($detail) { //a5
                $checkrows = '<input type="checkbox" name="selected_orders[]" class="slectedrows"  value="' . $detail->id . '" data-order-status="' . $detail->order_status . '" >';
                return $checkrows;
            })
            ->addColumn('suspectfroad', function ($detail) { //a6
                $customer_flag = 0;
                $order_customer = $detail->customer->first();
                if ($order_customer)
                    $customer_flag = $order_customer->flag;
                if (($detail->flag == 1 || $customer_flag == 1 || $detail->payment_status != 0) && $detail->order_status != 5 && $detail->order_status != 4 && $detail->order_status != 8) {
                    $custdet = "Flag";
                } elseif ($detail->flag == 2) {
                    $custdet = "Flagged";
                } else {
                    $custdet = "NoFlag";
                }
                if ($detail->flag == 3) {
                    $custdet = "NoFlag";
                }
                return $custdet;
            })
            ->addColumn('ordersinlast12months', function ($detail) { //a7
                $ordersinlast12months = 0;
                $order_customer = $detail->customer->first();
                if ($order_customer)
                    // $ordersinlast12months = OrderHelper::last_year_order_count($order_customer->id, $detail->id, $detail->created_at);
                return $ordersinlast12months;
            })
            ->addColumn('lastorderofproduct', function ($detail) { //a8
                $order_customer = $detail->customer->first();
                $allorders = '';
                if ($order_customer) {
                    foreach ((array)$detail->items as $item) {
                        $product = $item->product;
                        if ($product) {
                            $lastorderofproduct = OrderHelper::last_order_date($order_customer->id, $detail->id, $product->id, $detail->created_at);
                            if ($allorders != '') {
                                $allorders = $allorders . ' , ' . $lastorderofproduct;
                            } else {
                                $allorders = $lastorderofproduct;
                            }
                        }
                    }
                }
                return $allorders;
            })
            ->removeColumn('order_no', 'invoice_no', 'billing_last_name', 'flag', 'order_from', 'order_type', 'pharmacist', 'vat', 'shipping_city', 'shipping_postcode', 'courier_company', 'tell_gp', 'contacted_gp', 'consultation_type')
            ->escapeColumns([])
            ->make(false);
            //dd(\DB::getQueryLog());
    }

}
