<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use App\Models\Order;
use App\Models\Product;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Facades\DataTables;
use App\Exports\ProductsExport;
use App\Exports\ProductsLocationExport;
use App\Exports\ProductsExportAll;
use Maatwebsite\Excel\Facades\Excel;
/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class ProductController extends Controller
{
    protected $title = 'Products';
    protected $viewPath = 'products';
    protected $link = 'products';

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    /**
     * Show the Dashboard.
     */
    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'quantity-desc' => 'Quantity Desc',
            'quantity-asc' => 'Quantity Asc',
            'products.title-desc' => 'Product Name Desc',
            'products.title-asc' => 'Product Name Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;
        if($request->has('show_bmi'))
            $show_bmi = true;
        else 
            $show_bmi = false;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->show_bmi = $show_bmi;
        $criteria->id = '';
        if($request->has('download_option') && $request->download_option == 1)
        {
            return Excel::download(new ProductsExport($criteria), 'products.xlsx');
        }
        $counts = collect();
        $counts->total_products = Product::getBySalesCount($criteria);
        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/lists', compact('counts', 'page', 'request'));
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function lists(Request $request)
    {
        $sort = ['quantity', 'desc'];
        $detail = Product::select('products.id', 'products.title', 'products.slug', DB::raw('sum(d.quantity) as quantity'))
            ->join('order_details as d', 'd.product_id', 'products.id')
            ->join('orders as o', 'o.id', 'd.order_id')
            ->where('o.payment_status', 0)
            ->where('o.order_status', '!=', 5)
            ->where('o.order_from', 1);

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            $show_bmi = false;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('products.title', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'show_bmi') {
                    $show_bmi = true;
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
            if($show_bmi)
            {
                $detail->whereHas('categories', function($query) {
                    $query->where('product_categories.id', 103);
                });
            }
        }
        $detail->groupBy('products.id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->addColumn('action', function ($detail) {
                $action = '';
                $edit_url = url('products/' . $detail->id . '/' . $detail->slug);
                $action .= HtmlHelper::viewButton($edit_url);
                return $action;
            })
            ->removeColumn('id', 'slug')
            ->escapeColumns([])
            ->make(false);
    }

    public function productIndex(Request $request, $id, $slug)
    {
        $product = Product::findOrFail($id);
        if ($product->slug == $slug) {
            $page = collect();
            $page->title = 'Product - ' . $product->title;
            $page->link = url('products/' . $id . '/' . $slug);
            $page->sorts = [
                'orders.created_at-desc' => 'Latest Order',
                'orders.created_at-asc' => 'Oldest Order',
                'orders.id-desc' => 'Order ID Desc',
                'orders.id-asc' => 'Order ID Asc',
                'orders.billing_first_name-desc' => 'Customer Name Desc',
                'orders.billing_first_name-asc' => 'Customer Name Asc',
                'orders.billing_city-desc' => 'Location Desc',
                'orders.billing_city-asc' => 'Location Asc',
                'd.quantity-desc' => 'Quantity Desc',
                'd.quantity-asc' => 'Quantity Asc',
            ];

            $filter = $request->filter;
            $keyword = $request->keyword;
            if($request->has('show_bmi'))
                $show_bmi = true;
            else 
                $show_bmi = false;
            switch ($filter) {
                case 'today':
                    $start_date = '';
                    $end_date = Carbon::now();
                    break;
                case 'week':
                    $start_date = Carbon::now()->startOfWeek();
                    $end_date = Carbon::now();
                    break;
                case 'month':
                    $start_date = Carbon::now()->startOfMonth();
                    $end_date = Carbon::now();
                    break;
                case 'year':
                    $start_date = Carbon::now()->startOfYear();
                    $end_date = Carbon::now();
                    break;
                case 'lweek':
                    $start_date = Carbon::now()->subDays(7);
                    $end_date = Carbon::now();
                    break;
                case 'lmonth':
                    $start_date = Carbon::now()->subDays(30);
                    $end_date = Carbon::now();
                    break;
                case "lyear":
                    $start_date = Carbon::now()->subDays(365);
                    $end_date = Carbon::now();
                    break;
                default:
                    $start_date = '';
                    $end_date = '';
            }

            $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
            $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

            $criteria = collect();
            $criteria->id = $product->id ?? '';
            $criteria->keyword = $keyword ?? '';
            $criteria->type = $filter ?? '';
            $criteria->start_date = $start_date;
            $criteria->end_date = $end_date;
            $criteria->show_bmi = $show_bmi;
            $counts = collect();
            $counts->total_products = Product::getBySalesCount($criteria);
            $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
            $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

            return view($this->viewPath . '/product', compact('counts', 'page', 'request', 'product'));
        }
        abort('404');
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function productLists(Request $request, $id, $slug)
    {
        $sort = ['orders.created_at', 'desc'];
        $detail = Order::select('orders.id', 'orders.created_at', 'orders.billing_first_name', 'orders.billing_last_name','customers.email',DB::raw("IF(category_product.category_id = 103, customer_bmis.bmi_index , '') as bmi"), 'orders.billing_postcode', 'orders.billing_city', 'd.quantity')
            ->join('order_details as d', 'd.order_id', 'orders.id')
            ->leftJoin('order_customers','order_customers.order_id','=','orders.id')
            ->leftJoin('customers','order_customers.customer_id','=','customers.id')
            ->leftJoin('products','products.id','=','d.product_id')
            ->leftJoin('category_product','category_product.product_id','=','products.id')
           
            ->leftJoin('customer_bmis', function($query) 
            {
                $query->on('customers.id','=','customer_bmis.customer_id')
                ->whereRaw('customer_bmis.id IN (select MAX(a2.id) from customer_bmis as a2 join customers as u2 on u2.id = a2.customer_id group by u2.id)');
            })
            ->where('orders.payment_status', 0)
            ->where('orders.order_status', '!=', 5)
            ->where('orders.order_from', 1)
            ->where('d.product_id', $id);

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where(function ($query) use ($names) {
                        $query->where('orders.billing_first_name', 'like', "%{$names}%")
                            ->orWhere('orders.id', 'like', "%{$names}%")
                            ->orWhere('orders.billing_last_name', 'like', "%{$names}%")
                            ->orWhere('orders.billing_postcode', 'like', "%{$names}%")
                            ->orWhere('orders.billing_city', 'like', "%{$names}%");
                    });
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('orders.created_at', '>=', $from_date)
                    ->whereDate('orders.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('orders.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('orders.created_at', '=', $to_date);
            }
        }
        $detail->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('name', function ($detail) {
                return $detail->billing_first_name . ' ' . $detail->billing_last_name;
            })
            ->editColumn('postcode', function ($detail) {
                return $detail->billing_city . ', ' . $detail->billing_postcode;
            })
            ->editColumn('created_at', function ($detail) {
                return Carbon::parse($detail->created_at)->format('d/m/Y h:i a');
            })
            ->addColumn('action', function ($detail) {
                $product_url = url('sales/' . $detail->id);
                return HtmlHelper::showButton($product_url, 'Order Detail', 'link');
            })
            ->removeColumn('billing_last_name', 'billing_city')
            ->escapeColumns([])
            ->make(false);
    }

    public function locationSalesGraph(Request $request)
    {
        $filter = $request->filter;
        $keyword = $request->keyword;
        $status = $request->status;
        $id = $request->id ?? '';
        if($request->has('show_bmi'))
            $show_bmi = true;
        else 
            $show_bmi = false;

        $start_date = $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '';
        $end_date = $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '';

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? 'all';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->status = $status;
        $criteria->id = $id;
        $criteria->show_bmi = $show_bmi;
        return Product::getBySaleDay($criteria);
    }

    public function salesDataDashboard(Request $request)
    {
        $filter = $request->filter;
        $start_date = '';
        $end_date = '';
        if ($filter != '') {
            $start_date = Carbon::now();
            if ($filter == 'lweek') {
                $end_date = Carbon::now()->subWeek();
            } elseif ($filter == 'lmonth') {
                $end_date = Carbon::now()->subMonth();
            } elseif ($filter == 'lyear') {
                $end_date = Carbon::now()->subYear();
            }
        }

        $criteria = collect();
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        return Order::getProducts($criteria);
    }

    public function typeHeadSuggession(Request $request)
    {
        $query = $request->get('query');
        $filterResult = Product::where('title', 'LIKE', '%'. $query. '%')->pluck('title')->toArray();
        return response()->json($filterResult);
    }


    public function locations(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url('products');
      //  $page->link =url()->current();
        $page->sorts = [
            'quantity-desc' => 'Quantity Desc',
            'quantity-asc' => 'Quantity Asc',
            'products.title-desc' => 'Product Name Desc',
            'products.title-asc' => 'Product Name Asc',
        ];

        $filter = $request->sort;
        $keyword = $request->keyword;
        if($request->has('show_bmi'))
            $show_bmi = true;
        else 
            $show_bmi = false;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->show_bmi = $show_bmi;
        $criteria->id = '';
        if($request->has('download_option') && $request->download_option == 1)
        {
            
            return Excel::download(new ProductsLocationExport($criteria), 'products.xlsx');
        }
        $counts = collect();
        $counts->total_products = Product::getBySalesCount($criteria);
        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        $locations=DB::table('locations')->pluck('name','id');
 
        return view($this->viewPath . '/location', compact('counts', 'page', 'request','locations'));
    }
    
    
    
    
      /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function location_lists(Request $request)
    {
        
        $sort = ['quantity', 'desc'];
        $detail = Product::select('products.id', 'products.title', 'products.slug', DB::raw('sum(d.quantity) as quantity'),'l.name')
            ->join('order_details as d', 'd.product_id', 'products.id')
            ->leftJoin('locations as l', 'products.location_id', 'l.id')
            ->join('orders as o', 'o.id', 'd.order_id')
            ->where('o.payment_status', 0)
            ->where('o.order_status', '!=', 5)
            ->where('o.order_from', 1);
            
          
           
            
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            $show_bmi = false;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('products.title', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                    $detail->where('products.location_id', $sort);
                
                }
                
               
                if ($search['value'] != NULL && $search['name'] == 'show_bmi') {
                    $show_bmi = true;
                }
            }
            if ($from_date != NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '>=', $from_date)
                    ->whereDate('o.created_at', '<=', $to_date);
            } elseif ($from_date != NULL && $to_date == NULL) {
                $detail->whereDate('o.created_at', '=', $from_date);
            } elseif ($from_date == NULL && $to_date != NULL) {
                $detail->whereDate('o.created_at', '=', $to_date);
            }
            if($show_bmi)
            {
                $detail->whereHas('categories', function($query) {
                    $query->where('product_categories.id', 103);
                });
            }
        }
    
        $detail->groupBy('products.id');
//            ->orderBy($sort[0], $sort[1]);
            
     
        return DataTables::of($detail)
            ->addColumn('action', function ($detail) {
                $action = '';
                $edit_url = url('products/' . $detail->id . '/' . $detail->slug);
                $action .= HtmlHelper::viewButton($edit_url);
                return $action;
            })
            ->removeColumn('id', 'slug')
             ->escapeColumns([])
            ->make(false);
    }


  public function viewAll(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url('products/all-lists');
        $page->sorts = [
            'stock_quantity-desc' => 'Quantity Desc',
            'stock_quantity-asc' => 'Quantity Asc',
            'products.title-desc' => 'Product Name Desc',
            'products.title-asc' => 'Product Name Asc',
        ];

        $filter = $request->filter;
        $keyword = $request->keyword;
        if($request->has('show_otc'))
            $show_otc = true;
        else 
            $show_otc = false;
            
            
        if($request->has('show_in_stock'))
            $show_in_stock = true;
        else 
            $show_in_stock = false;
            
            
        if($request->has('show_out_stock'))
            $show_out_stock = true;
        else 
            $show_out_stock = false;
            

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }

        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;

        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';
        $criteria->start_date = $start_date;
        $criteria->end_date = $end_date;
        $criteria->show_otc = $show_otc;
        $criteria->id = '';
        if($request->has('download_option') && $request->download_option == 1)
        {
            return Excel::download(new ProductsExportAll($criteria), 'products.xlsx');
        }
        $counts = collect();
        $counts->total_products = Product::getBySalesCount($criteria);
        $request->start_date = $start_date != '' ? $start_date->format('d/m/Y') : '';
        $request->end_date = $end_date != '' ? $end_date->format('d/m/Y') : '';

        return view($this->viewPath . '/all_lists', compact('counts', 'page', 'request'));
    }
    
    
        public function allLists(Request $request)
    {
        $sort = ['stock_quantity', 'asc'];
        $detail = Product::select('products.id', 'products.title', 'products.slug','stock_quantity');
        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            $show_otc = false;
            $show_in_stock = false;
            $show_out_stock = false;
            
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('products.title', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'start_date') {
                    $from_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'end_date') {
                    $to_date = Carbon::createFromFormat('d/m/Y', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
                if ($search['value'] != NULL && $search['name'] == 'show_otc') {
                    $show_otc = true;
                }
                
                
                if ($search['value'] != NULL && $search['name'] == 'show_in_stock') {
                    $show_in_stock = true;
                }
                
                if ($search['value'] != NULL && $search['name'] == 'show_out_stock') {
                    $show_out_stock = true;
                }
                
            }
           
            if($show_otc)
            {
                $detail->where("is_otc",1);
            }
            
            
            if($show_in_stock)
            {
                  
           
                $detail->where("stock_quantity",">",0);
            }
            
            
            if($show_out_stock)
            {
                $detail->where("stock_quantity","<",1);
               
            }
            
            
          
            
            
        }
        $detail->groupBy('products.id')
            ->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->addColumn('action', function ($detail) {
                $action = '';
                $edit_url = url('products/' . $detail->id . '/' . $detail->slug);
                $action .= HtmlHelper::viewButton($edit_url);
                return $action;
            })
            ->removeColumn('id', 'slug')
            ->escapeColumns([])
            ->make(false);
    }

}
