<?php

namespace App\Http\Controllers;

use App\Helpers\HtmlHelper;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Vansteen\Sendinblue\Facades\Sendinblue;

use GuzzleHttp\Client;
use Yajra\DataTables\Facades\DataTables;

class SendinBlueController extends Controller
{

    protected $title = 'Email';
    protected $viewPath = 'emails';
    protected $link = 'emails';
    protected $orderStatus = ['0' => 'Same Day Delivery', '1' => 'Next Day Delivery', '2' => 'Normal Delivery', '3' => 'Pending', '4' => 'Completed'];


    public function index(Request $request)
    {

        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->order_status = $this->orderStatus;


        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
        ];





        // Configure API keys authorization according to the config file
        $config = Sendinblue::getConfiguration();

        // Uncomment below to setup prefix (e.g. Bearer) for API keys, if needed
        // $config->setApiKeyPrefix('api-key', 'Bearer');
        // $config->setApiKeyPrefix('partner-key', 'Bearer');

        $apiInstance = new \SendinBlue\Client\Api\ListsApi(
            // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
            // This is optional, `GuzzleHttp\Client` will be used as default.
            new Client(),


            $config
        );

        $detail = $apiInstance->getLists();
        $campaigns = array();
        foreach ($detail["lists"] as $key => $data) {

            $folderinstance = $apiInstance->getList($data["id"]);

            $campaigns[$key] = array(
                "id" => $folderinstance['id'],

                "name" => $folderinstance['name'],
                "uniqueSubscribers" => $folderinstance['uniqueSubscribers'],
                "totalBlacklisted" => $folderinstance['totalBlacklisted'],
                "totalSubscribers" => $folderinstance['totalSubscribers'],


            );
        }



        return view($this->viewPath . '/lists', compact('page', 'request', 'campaigns'));
    }

    public function list()
    {

        // Configure API keys authorization according to the config file
        $config = Sendinblue::getConfiguration();

        // Uncomment below to setup prefix (e.g. Bearer) for API keys, if needed
        // $config->setApiKeyPrefix('api-key', 'Bearer');
        // $config->setApiKeyPrefix('partner-key', 'Bearer');

        $apiInstance = new \SendinBlue\Client\Api\ListsApi(
            // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
            // This is optional, `GuzzleHttp\Client` will be used as default.
            new Client(),


            $config
        );

        $detail = $apiInstance->getLists();
        $campaigns = array();
        foreach ($detail["lists"] as $key => $data) {

            $folderinstance = $apiInstance->getList($data["id"]);

            $campaigns[$key] = array(
                "id" => $folderinstance['id'],

                "name" => $folderinstance['name'],
                "uniqueSubscribers" => $folderinstance['uniqueSubscribers'],
                "totalBlacklisted" => $folderinstance['totalBlacklisted'],
                "totalSubscribers" => $folderinstance['totalSubscribers'],


            );
        }


        return DataTables::of($campaigns)
            ->addIndexColumn()->editColumn('name', function ($detail) {
                return $detail['name'];
            })->editColumn('uniqueSubscribers', function ($detail) {
                return $detail['uniqueSubscribers'];
            })->editColumn('totalBlacklisted', function ($detail) {
                return $detail['totalBlacklisted'];
            })->editColumn('totalSubscribers', function ($detail) {
                return $detail['totalSubscribers'];
            })->addColumn('action', function ($detail) { //a4


                $sale_url = url('emails/view/' . $detail['id']);

                $action = HtmlHelper::showButton($sale_url, 'View', 'eye');

                return $action;
            })->removeColumn('folderId')
            ->orderColumn('totalSubscribers')
            ->escapeColumns([])
            ->make(false);

        // return DataTables::of($detail)->editColumn('shipping_ref_no', function ($detail) { 

        //     return $detail->shipping_ref_no;
        // })  ->removeColumn('order_no', 'invoice_no', 'billing_last_name', 'flag', 'order_from', 'order_type', 'pharmacist', 'vat', 'shipping_city', 'shipping_postcode', 'courier_company', 'tell_gp', 'contacted_gp', 'consultation_type')
        // ->escapeColumns([])
        // ->make(false);

    }

    public function view(Request $request, $id)
    {


        $page = collect();
        $page->title = "Campaign - ";

        $page->link = url('emails/Campaign');
        $page->order_status = $this->orderStatus;


        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
        ];




        // Configure API keys authorization according to the config file
        $config = Sendinblue::getConfiguration();

        // Uncomment below to setup prefix (e.g. Bearer) for API keys, if needed
        // $config->setApiKeyPrefix('api-key', 'Bearer');
        // $config->setApiKeyPrefix('partner-key', 'Bearer');

        $apiInstance = new \SendinBlue\Client\Api\ListsApi(
            // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
            // This is optional, `GuzzleHttp\Client` will be used as default.
            new Client(),


            $config
        );


        $data = $apiInstance->getList($id);

        $data["clickers"] = 0;
        $data["viewed"] = 0;
        $data["unsubscriptions"] = 0;

        foreach ($data["campaignStats"] as $campaign) {



            $data["clickers"] = $data["clickers"] + $campaign["stats"]["clickers"];
            $data["viewed"] = $data["viewed"] + $campaign["stats"]["viewed"];
            $data["unsubscriptions"] = $data["unsubscriptions"] + $campaign["stats"]["unsubscriptions"];
        }
        return view($this->viewPath . '/view_campaign', compact('page', 'request', 'data'));
    }

    public function CampaignSubscribers(Request $request, $id)
    {



        // Configure API keys authorization according to the config file
        $config = Sendinblue::getConfiguration();

        // Uncomment below to setup prefix (e.g. Bearer) for API keys, if needed
        // $config->setApiKeyPrefix('api-key', 'Bearer');
        // $config->setApiKeyPrefix('partner-key', 'Bearer');

        $apiInstance = new \SendinBlue\Client\Api\ListsApi(
            // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
            // This is optional, `GuzzleHttp\Client` will be used as default.
            new Client(),


            $config
        );

        $limit = $request->length;
        $offset = $request->start;
        $sort = "DESC";


        $detail = $apiInstance->getContactsFromList($id, null, $limit, $offset, $sort);


        return DataTables::of($detail["contacts"])
            ->addIndexColumn()->addColumn('name', function ($detail) {
                return @$detail['attributes']->FIRSTNAME . " " . @$detail['attributes']->LASTNAME;
            })->addColumn('emails', function ($detail) {
                // dd($detail);
                return @$detail["email"];
            })->addColumn('phone', function ($detail) {


                return @$detail['attributes']->CONTACT_NUMBER;
            })->addColumn('age', function ($detail) {
                return @$detail['attributes']->AGE;
            })->addColumn('email_black_list', function ($detail) {
                return @$detail["emailBlacklisted"];
            })->addColumn('sms_black_list', function ($detail) {


                return @$detail["smsBlacklisted"];
            })->addColumn('created_at', function ($detail) {
                return @$detail["createdAt"];
            })->addColumn('modified_at', function ($detail) {

                return @$detail["modifiedAt"];
            })
            //    ->addColumn('order_date', function ($detail) { 
            //     return @$detail['attributes']->ORDER_DATE;


            //    })
            ->addColumn('action', function ($detail) { //a4


                $sale_url = url('emails/contact-view/' . $detail['id']);

                $action = HtmlHelper::showButton($sale_url, 'View', 'eye');

                return $action;
            })



            // ->editColumn('uniqueSubscribers', function ($detail) { 
            //     return $detail['uniqueSubscribers'];
            // })->editColumn('totalBlacklisted', function ($detail) { 
            //     return $detail['totalBlacklisted'];
            // }) ->editColumn('totalSubscribers', function ($detail) { 
            //     return $detail['totalSubscribers'];
            // })   ->addColumn('action', function ($detail) { //a4


            //     $sale_url = url('emails/view/' . $detail['id']);

            //         $action = HtmlHelper::showButton($sale_url, 'View', 'eye');

            //     return $action;
            // })
            ->removeColumn(
                'attributes',
                'listIds',
                'id',
                'createdAt',
                'modifiedAt',
                'ORDER_DATE',
                'emailBlacklisted',
                'smsBlacklisted'

            )->setTotalRecords($detail["count"])
            ->escapeColumns([])
            ->skipPaging()
            ->make(false);

        // createdAt: "2021-12-13T19:38:08.332+00:00"
        // email: "eliza_magnusen@hotmail.com"
        // emailBlacklisted: false
        // id: 154294


    }

    public function contactView(Request $request, $id)
    {




        $page = collect();
        $page->title = "Contact - ";

        $page->link = url('emails/contact-view');
        $page->order_status = $this->orderStatus;


        $page->start_date = Carbon::now()->subDays(29)->format('d/m/Y');
        $page->end_date = Carbon::now()->format('d/m/Y');


        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
        ];



        // Configure API keys authorization according to the config file
        $config = Sendinblue::getConfiguration();

        // Uncomment below to setup prefix (e.g. Bearer) for API keys, if needed
        // $config->setApiKeyPrefix('api-key', 'Bearer');
        // $config->setApiKeyPrefix('partner-key', 'Bearer');

        $apiInstance = new \SendinBlue\Client\Api\ContactsApi(
            // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
            // This is optional, `GuzzleHttp\Client` will be used as default.
            new Client(),


            $config
        );



        try {
            $data = $apiInstance->getContactInfo($id);
        } catch (Exception $e) {
            echo 'Exception when calling ContactsApi->getContactInfo: ', $e->getMessage(), PHP_EOL;
        }

        return view($this->viewPath . '/view_contact', compact('page', 'request', 'data'));
    }


    public function userEmailList(Request $request, $id)
    {


        $config = Sendinblue::getConfiguration();

        // Uncomment below to setup prefix (e.g. Bearer) for API keys, if needed
        // $config->setApiKeyPrefix('api-key', 'Bearer');
        // $config->setApiKeyPrefix('partner-key', 'Bearer');

        $apiInstance = new \SendinBlue\Client\Api\ContactsApi(
            // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
            // This is optional, `GuzzleHttp\Client` will be used as default.
            new Client(),


            $config
        );

        try {




            $result = $apiInstance->getContactInfo($id);

            $apiInstance = new \SendinBlue\Client\Api\TransactionalEmailsApi(
                // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
                // This is optional, `GuzzleHttp\Client` will be used as default.
                new Client(),
                $config
            );

            $days = null;
            $event = null;
            $tags = null;


            $email = $result["email"]; // string | Mandatory if templateId and messageId are not passed in query filters. Email address to which transactional email has been sent.
            $templateId = NULL; // int | Mandatory if email and messageId are not passed in query filters. Id of the template that was used to compose transactional email.
            $messageId = NULL; // string | Mandatory if templateId and email are not passed in query filters. Message ID of the transactional email sent.
            $startDate = Carbon::createFromFormat('d/m/Y', $request->startDate)->format("Y-m-d"); // string | Mandatory if endDate is used. Starting date (YYYY-MM-DD) from which you want to fetch the list. Maximum time period that can be selected is one month.
            $endDate = Carbon::createFromFormat('d/m/Y', $request->endDate)->format("Y-m-d"); // string | Mandatory if startDate is used. Ending date (YYYY-MM-DD) till which you want to fetch the list. Maximum time period that can be selected is one month.
            $sort = $request->sort; // string | Sort the results in the ascending/descending order of record creation. Default order is **descending** if `sort` is not passed
            $limit = $request->length;
            $offset = $request->start;
 
            try {

                // $result = $apiInstance->getEmailEventReport($limit, $offset, $startDate, $endDate, $days, $email, $event, $tags, $messageId, $templateId, $sort);
                // dump($result);


                // var_dump($offset.$startDate.$endDate.$days.$email.$event.$tags.$messageId.$templateId.$sort);
                // die;
                $detail = $apiInstance->getEmailEventReport($limit, $offset, $startDate, $endDate, $days = null, $email, $event = null, $tags = null, $messageId = null, $templateId = null, $sort);
//                 $detail = $apiInstance->getTransacEmailsList($email,$templateId,$messageId,$startDate=null,$endDate=null , $sort,$limit, $offset);
// dd($detail);


                // $detail = $apiInstance->getTransacEmailsList($limit, $offset, $startDate, $endDate, $days = null, $email, $event = null, $tags = null, $messageId = null, $templateId = null, $sort);

                if($detail["events"]){
                return DataTables::of($detail["events"])
                    ->addIndexColumn()->addColumn('subject', function ($detail) {
                        return @$detail['subject'];
                    })->addColumn('event', function ($detail) {
                        // dd($detail);
                        return @$detail["event"];
                    })->addColumn('date', function ($detail) {
                        return @$detail["date"];
                    })
                    ->addColumn('action', function ($detail) { //a4


                        $sale_url = url('emails/contact-view/email-view/' . $detail['messageId']);

                        $action = HtmlHelper::showButton($sale_url, 'View', 'eye');

                        return $action;
                    })
                    ->removeColumn('reason', 'tag', 'ip', 'link', 'from', 'templateId', 'age', 'phone')->setTotalRecords($detail["count"])
                    ->escapeColumns([])
                    ->skipPaging()
                    ->make(false);



                }else{
                    return DataTables::of(array())->escapeColumns([])
                    ->skipPaging()
                    ->make(false);
                } 


            } catch (Exception $e) {
                echo 'Exception when calling TransactionalEmailsApi->getTransacEmailsList: ', $e->getMessage(), PHP_EOL;
            }



            // dd($result);



        } catch (Exception $e) {
            echo 'Exception when calling ContactsApi->getContactInfo: ', $e->getMessage(), PHP_EOL;
        }
    }



    public function viewCampaign(Request $request, $id)
    {


        $page = collect();
        $page->title = "Campaign - ";

        $page->link = url('emails/Campaign');
        $page->order_status = $this->orderStatus;


        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
        ];



        // Configure API keys authorization according to the config file
        $config = Sendinblue::getConfiguration();

        // Uncomment below to setup prefix (e.g. Bearer) for API keys, if needed
        // $config->setApiKeyPrefix('api-key', 'Bearer');
        // $config->setApiKeyPrefix('partner-key', 'Bearer');

        $apiInstance = new \SendinBlue\Client\Api\EmailCampaignsApi(
            // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
            // This is optional, `GuzzleHttp\Client` will be used as default.
            new Client(),


            $config
        );

        $data = $apiInstance->getEmailCampaign($id);




        return view($this->viewPath . '/detailed_view_campaign', compact('page', 'request', 'data'));
    }




    public function showContactListPage(Request $request)
    {


        $page = collect();
        $page->title = "Contacts - ";

        $page->link = url('emails/contacts');
        $page->order_status = $this->orderStatus;


        $page->sorts = [
            'created_at-desc' => 'Latest Order',
            'created_at-asc' => 'Oldest Order',
        ];



        return view($this->viewPath . '/contacts', compact('page', 'request'));
    }




    public function getContactList(Request $request)
    {

        // Configure API keys authorization according to the config file
        $config = Sendinblue::getConfiguration();

        $apiInstance = new \SendinBlue\Client\Api\ContactsApi(
            // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
            // This is optional, `GuzzleHttp\Client` will be used as default.
            new Client(),
            $config
        );

        $limit = $request->length;
        $offset = $request->start;
        $sort = "DESC";
        $modifiedSince = "";

        // $detail = $apiInstance->getContactsFromList($id,null, $limit, $offset, $sort);
        $detail = $apiInstance->getContacts($limit, $offset, $modifiedSince, $sort);

        return DataTables::of($detail["contacts"])
            ->addIndexColumn()->addColumn('name', function ($detail) {
                return @$detail['attributes']->FIRSTNAME . " " . @$detail['attributes']->LASTNAME;
            })->addColumn('emails', function ($detail) {
                // dd($detail);
                return @$detail["email"];
            })->addColumn('phone', function ($detail) {


                return @$detail['attributes']->CONTACT_NUMBER;
            })->addColumn('age', function ($detail) {
                return @$detail['attributes']->AGE;
            })->addColumn('email_black_list', function ($detail) {
                return @$detail["emailBlacklisted"];
            })->addColumn('sms_black_list', function ($detail) {


                return @$detail["smsBlacklisted"];
            })->addColumn('created_at', function ($detail) {
                return @$detail["createdAt"];
            })->addColumn('modified_at', function ($detail) {

                return @$detail["modifiedAt"];
            })
            //   ->addColumn('order_date', function ($detail) { 
            //    return @$detail['attributes']->ORDER_DATE;


            //   })

            ->addColumn('action', function ($detail) { //a4

                $sale_url = url('emails/contact-view/' . $detail['id']);
                $action = HtmlHelper::showButton($sale_url, 'View', 'eye');
                return $action;
            })
            ->removeColumn(
                'attributes',
                'listIds',
                'id',
                'createdAt',
                'modifiedAt',
                'ORDER_DATE',
                'emailBlacklisted',
                'smsBlacklisted'

            )->setTotalRecords($detail["count"])
            ->escapeColumns([])
            ->skipPaging()
            ->make(false);

        // createdAt: "2021-12-13T19:38:08.332+00:00"
        // email: "eliza_magnusen@hotmail.com"
        // emailBlacklisted: false
        // id: 154294


    }




    public function userEmailView($id)
    {




        $config = Sendinblue::getConfiguration();

        try {

            $apiInstance = new \SendinBlue\Client\Api\TransactionalEmailsApi(
                // If you want use custom http client, pass your client which implements `GuzzleHttp\ClientInterface`.
                // This is optional, `GuzzleHttp\Client` will be used as default.
                new Client(),
                $config
            );
            echo "s";
            $result = $apiInstance->getScheduledEmailByMessageId($id);
            dd($result);
            echo "R";
        } catch (Exception $e) {
            dd($e);
        }
    }
}
