<?php

namespace App\Http\Controllers;

use App\Models\Page;
use App\Models\Product;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Yajra\DataTables\Facades\DataTables;
use App\Exports\StocksExport;
use Maatwebsite\Excel\Facades\Excel;

/**
 * Class HomeController
 * @package App\Http\Controllers\Admin
 */
class StockController extends Controller
{
    protected $title = 'Stocks';
    protected $viewPath = 'stocks';
    protected $link = 'stocks';

    /**
     * Create a new controller instance.
     */
    public function __construct()
    {
    }

    /**
     * Show the Dashboard.
     */
    public function index(Request $request)
    {
        $page = collect();
        $page->title = $this->title;
        $page->link = url($this->link);
        $page->sorts = [
            'title-asc' => 'Product Name Asc',
            'title-desc' => 'Product Name Desc',
            'stock_quantity-asc' => 'Stock Asc',
            'stock_quantity-desc' => 'Stock Desc',
        ];

        $filter = $request->filter;

        switch ($filter) {
            case 'today':
                $start_date = '';
                $end_date = Carbon::now();
                break;
            case 'week':
                $start_date = Carbon::now()->startOfWeek();
                $end_date = Carbon::now();
                break;
            case 'month':
                $start_date = Carbon::now()->startOfMonth();
                $end_date = Carbon::now();
                break;
            case 'year':
                $start_date = Carbon::now()->startOfYear();
                $end_date = Carbon::now();
                break;
            case 'lweek':
                $start_date = Carbon::now()->subDays(7);
                $end_date = Carbon::now();
                break;
            case 'lmonth':
                $start_date = Carbon::now()->subDays(30);
                $end_date = Carbon::now();
                break;
            case "lyear":
                $start_date = Carbon::now()->subDays(365);
                $end_date = Carbon::now();
                break;
            default:
                $start_date = '';
                $end_date = '';
        }


        $start_date = $filter == '' ? $request->start_date != '' ? Carbon::createFromFormat('d/m/Y', $request->start_date) : '' : $start_date;
        $end_date = $filter == '' ? $request->end_date != '' ? Carbon::createFromFormat('d/m/Y', $request->end_date) : '' : $end_date;


        $criteria = collect();
        $criteria->keyword = $keyword ?? '';
        $criteria->type = $filter ?? '';

        $criteria->sorts = array("stock_quantity", "desc");

        if ($request->keyword != NULL) {
            $names = strtolower($request->keyword);
            // $detail->where('title', 'like', "%{$names}%");
            $criteria->keyword = $names;
        }

        if (isset($request->sort)) {

            if ($request->sort != NULL) {
                $criteria->sorts  = explode('-', $request->sort);
            }
        }

        if ($request->has('download_option') && $request->download_option == 1) {
            return Excel::download(new StocksExport($criteria), 'stocks.xlsx');
        }

        return view($this->viewPath, compact('page', 'request'));
    }

    /**
     * Display a listing of the resource in datatable.
     * @throws \Exception
     */
    public function lists(Request $request)
    {
        $sort = ['stock_quantity', 'asc'];
        $detail = Product::where('stock_quantity', '>=', 0)
            ->select('title', 'stock_quantity');

        if (isset($request->form)) {
            $from_date = NULL;
            $to_date = NULL;
            foreach ($request->form as $search) {
                if ($search['value'] != NULL && $search['name'] == 'keyword') {
                    $names = strtolower($search['value']);
                    $detail->where('title', 'like', "%{$names}%");
                }
                if ($search['value'] != NULL && $search['name'] == 'sort') {
                    $sort = explode('-', $search['value']);
                }
            }
        }
        $detail->orderBy($sort[0], $sort[1]);
        return DataTables::of($detail)
            ->editColumn('stock_quantity', function ($detail) {
                $data = $detail->stock_quantity;
                if ($detail->stock_quantity <= 0) {
                    $data = '<span style="color: #ff0000">' . $detail->stock_quantity . '</span>';
                }
                return $data;
            })
            ->escapeColumns([])
            ->make(false);
    }
}
