<?php

namespace App\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;

class AbandonedOrder extends Model
{
    public function order()
    {
        return $this->belongsTo(Order::class);
    }

    public function getCreatedAtAttribute($value)
    {
        return Carbon::parse($value)->format('d/m/Y h:i A');
    }

    public static function getBySaleDay($criteria)
    {
        $keyword = $criteria->keyword;
        $filter = $criteria->type;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;

        $date = [];
        $result = [];
        $format = [];

        if ($start_date != '' && $end_date == '') {
            $date[] = $start_date;
        } elseif ($start_date == '' && $end_date != '') {
            $date[] = $end_date;
        } else {
            $start_date = $start_date == '' ? Carbon::now()->subDay(15) : $start_date;
            $end_date = $end_date == '' ? Carbon::now()->addDay() : $end_date->addDay();

            $formated_date = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1D'),
                new \DateTime($end_date)
            );
            foreach ($formated_date as $fd) {
                $date[] = $fd;
            }
            $date = array_reverse($date);
        }

        foreach ($date as $row) {
            $result[] = self::join('orders as o', 'o.id', 'abandoned_orders.order_id')
                ->when($keyword, function ($query) use ($keyword) {
                    return $query->where(function ($q) use ($keyword) {
                        $q->where('billing_first_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_last_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_city', 'like', "%{$keyword}%")
                            ->orWhere('billing_postcode', 'like', "%{$keyword}%")
                            ->orWhere('order_no', 'like', "%{$keyword}%");
                    });
                })
                ->where('o.payment_status', 1)
                ->whereDate('abandoned_orders.created_at', '=', $row)
                ->count();
            $format[] = Carbon::parse($row)->format('d M');
        }

        return json_encode(['date' => $format, 'result' => $result]);
    }

    public static function getByRevenueDay($criteria)
    {
        $keyword = $criteria->keyword;
        $filter = $criteria->type;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;

        $date = [];
        $result = [];
        $format = [];

        if ($start_date != '' && $end_date == '') {
            $date[] = $start_date;
        } elseif ($start_date == '' && $end_date != '') {
            $date[] = $end_date;
        } else {
            $start_date = $start_date == '' ? Carbon::now()->subDay(15) : $start_date;
            $end_date = $end_date == '' ? Carbon::now()->addDay() : $end_date->addDay();

            $formated_date = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1D'),
                new \DateTime($end_date)
            );
            foreach ($formated_date as $fd) {
                $date[] = $fd;
            }
            $date = array_reverse($date);
        }

        foreach ($date as $row) {
            $result[] = self::join('orders as o', 'o.id', 'abandoned_orders.order_id')
                ->when($keyword, function ($query) use ($keyword) {
                    return $query->where(function ($q) use ($keyword) {
                        $q->where('billing_first_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_last_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_city', 'like', "%{$keyword}%")
                            ->orWhere('billing_postcode', 'like', "%{$keyword}%")
                            ->orWhere('order_no', 'like', "%{$keyword}%");
                    });
                })
                ->where('o.payment_status', 1)
                ->whereDate('abandoned_orders.created_at', '=', $row)
                ->sum('amount_to_pay');
            $format[] = Carbon::parse($row)->format('d M');

        }
        return json_encode(['date' => $format, 'result' => $result]);
    }
}
