<?php

namespace App\Models;

use Carbon\Carbon;
use Carbon\CarbonPeriod;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Brand extends Model
{
    public static function getBySales($criteria)
    {
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;
        $date = [];
        $result = [];
        $format = [];
        if (($start_date == '' && $end_date == '') || $filter == 'all') {
            $check_date = Carbon::now()->format('Y-m-d');
            $date[] = $check_date;
            for ($i = 1; $i <= 14; $i++) {
                $check_date = Carbon::now()->subDays($i)->format('Y-m-d');
                $date[] = $check_date;
            }
        } elseif ($filter == 'today') {
            $check_date = Carbon::now()->format('Y-m-d');
            $date[] = $check_date;
        } else {
            if ($start_date != '' && $end_date == '') {
                $check_date = Carbon::parse($start_date)->format('Y-m-d');
                $date[] = $check_date;
            } elseif ($start_date == '' && $end_date != '') {
                $check_date = Carbon::parse($end_date)->format('Y-m-d');
                $date[] = $check_date;
            } elseif ($start_date != '' && $end_date != '') {
                $period = CarbonPeriod::create($start_date, $end_date);
                foreach ($period as $dates) {
                    $date[] = $dates->format('Y-m-d');
                }
            } else {

            }
        }
        foreach ($date as $row) {
            $result[] = DB::table('orders as o')
                ->join('order_carts as oc', 'o.id', 'oc.order_id')
                ->join('products as p', 'p.id', 'oc.product_id')
                ->join('contents as c', 'p.content_id', 'c.id')
                ->where('o.payment', 1)
                ->when($keyword, function ($query) use ($keyword) {
                    $query->where('c.title', 'like', "%{$keyword}%");
                    return $query;
                })
                ->whereDate('o.datetime', '=', $row)
                ->sum('oc.qty');
            $format[] = Carbon::parse($row)->format('d M');
        }
        return json_encode(['date' => $format, 'result' => $result]);
    }

    public static function getBySalesCount($criteria)
    {
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        return DB::table('orders as o')
            ->join('order_carts as oc', 'o.id', 'oc.order_id')
            ->join('products as p', 'p.id', 'oc.product_id')
            ->join('contents as c', 'p.content_id', 'c.id')
            ->where('o.payment', 1)
            ->when($keyword, function ($q) use ($keyword) {
                return $q->where('c.title', 'like', "%{$keyword}%");
            })
            ->when($start_date == '' && $end_date != '', function ($q) use ($end_date) {
                return $q->whereDate('o.datetime', '=', $end_date);
            })
            ->when($start_date != '' && $end_date == '', function ($q) use ($start_date) {
                return $q->whereDate('o.datetime', '=', $start_date);
            })
            ->when($start_date != '' && $end_date != '', function ($q) use ($start_date, $end_date) {
                return $q->whereBetween('o.datetime', [$start_date, $end_date]);
            })->sum('oc.qty');
    }

    public static function getByProductSales($criteria)
    {
        $id = $criteria->id;
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;
        $date = [];
        $result = [];
        $format = [];
        if (($start_date == '' && $end_date == '') || $filter == 'all') {
            $check_date = Carbon::now()->format('Y-m-d');
            $date[] = $check_date;
            for ($i = 1; $i <= 14; $i++) {
                $check_date = Carbon::now()->subDays($i)->format('Y-m-d');
                $date[] = $check_date;
            }
        } elseif ($filter == 'today') {
            $check_date = Carbon::now()->format('Y-m-d');
            $date[] = $check_date;
        } else {
            if ($start_date != '' && $end_date == '') {
                $check_date = Carbon::parse($start_date)->format('Y-m-d');
                $date[] = $check_date;
            } elseif ($start_date == '' && $end_date != '') {
                $check_date = Carbon::parse($end_date)->format('Y-m-d');
                $date[] = $check_date;
            } elseif ($start_date != '' && $end_date != '') {
                $period = CarbonPeriod::create($start_date, $end_date);
                foreach ($period as $dates) {
                    $date[] = $dates->format('Y-m-d');
                }
            } else {

            }
        }
        foreach ($date as $row) {
            $result[] = DB::table('orders as o')
                ->join('order_carts as oc', 'o.id', 'oc.order_id')
                ->leftJoin('users as u', 'u.id', 'o.user_id')
                ->join('products as p', 'p.id', 'oc.product_id')
                ->join('contents as c', 'p.content_id', 'c.id')
                ->where('o.payment', 1)
                ->when($id, function ($query) use ($id) {
                    $query->where('c.id', $id);
                    return $query;
                })
                ->when($keyword, function ($q) use ($keyword) {
                    return $q->where(function ($query) use ($keyword) {
                        $query->where('u.name', 'like', "%{$keyword}%")
                            ->orWhere('o.id', 'like', "%{$keyword}%")
                            ->orWhere('u.lname', 'like', "%{$keyword}%")
                            ->orWhere('o.postcode', 'like', "%{$keyword}%");
                    });
                })
                ->whereDate('o.datetime', '=', $row)
                ->sum('oc.qty');
            $format[] = Carbon::parse($row)->format('d M');
        }
        return json_encode(['date' => $format, 'result' => $result]);
    }

    public static function getByProductSalesCount($criteria)
    {
        $id = $criteria->id;
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        return DB::table('orders as o')
            ->join('order_carts as oc', 'o.id', 'oc.order_id')
            ->leftJoin('users as u', 'u.id', 'o.user_id')
            ->join('products as p', 'p.id', 'oc.product_id')
            ->join('contents as c', 'p.content_id', 'c.id')
            ->where('o.payment', 1)
            ->when($id, function ($q) use ($id) {
                return $q->where('c.id', $id);
            })
            ->when($keyword, function ($q) use ($keyword) {
                return $q->where(function ($query) use ($keyword) {
                    $query->where('u.name', 'like', "%{$keyword}%")
                        ->orWhere('o.id', 'like', "%{$keyword}%")
                        ->orWhere('u.lname', 'like', "%{$keyword}%")
                        ->orWhere('o.postcode', 'like', "%{$keyword}%");
                });
            })
            ->when($start_date == '' && $end_date != '', function ($q) use ($end_date) {
                return $q->whereDate('o.datetime', '=', $end_date);
            })
            ->when($start_date != '' && $end_date == '', function ($q) use ($start_date) {
                return $q->whereDate('o.datetime', '=', $start_date);
            })
            ->when($start_date != '' && $end_date != '', function ($q) use ($start_date, $end_date) {
                return $q->whereBetween('o.datetime', [$start_date, $end_date]);
            })->sum('oc.qty');
    }
}
