<?php

namespace App\Models;

use Carbon\Carbon;
use GuzzleHttp\Client;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class OtcOrder extends Model
{
    protected $table = 'orders';
   
    public function scopePaid($query)
    {
        return $query->where('orders.payment_status', 0);
    }

    public function scopeOnline($query)
    {
        return $query->where('orders.order_from', 1);
    }

    public function scopeActive($query)
    {
        return $query->where('orders.order_status', '!=', 5)->where('orders.order_status','!=',8);
    }
 


    public static function getBySaleDay($criteria)
    {
        $keyword = $criteria->keyword;
        $filter = $criteria->type;
        $status = $criteria->status;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;

        $date = [];
        $result = [];
        $format = [];

        if ($start_date != '' && $end_date == '') {
            $date[] = $start_date;
        } elseif ($start_date == '' && $end_date != '') {
            $date[] = $end_date;
        } else {
            $start_date = $start_date == '' ? Carbon::now()->subDay(15) : $start_date;
            $end_date = $end_date == '' ? Carbon::now()->addDay() : $end_date->addDay();

            $formated_date = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1D'),
                new \DateTime($end_date)
            );
            foreach ($formated_date as $fd) {
                $date[] = $fd;
            }
            $date = array_reverse($date);
        }

        foreach ($date as $row) {
            $result[] = self::leftJoin('order_details as d', 'orders.id', 'd.order_id')
            ->leftJoin('products as p', 'd.product_id', 'p.id')
            ->where('p.is_otc',1)->paid()->active()
                ->when($keyword, function ($query) use ($keyword) {
                    return $query->where(function ($q) use ($keyword) {
                        $q->where('billing_first_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_last_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_city', 'like', "%{$keyword}%")
                            ->orWhere('billing_postcode', 'like', "%{$keyword}%")
                            ->orWhere('invoice_no', 'like', "%{$keyword}%")
                            ->orWhere('order_no', 'like', "%{$keyword}%");
                    });
                })
                ->when($status!=null, function ($query) use ($status) {
              
                    return $query->where('orders.order_status', $status);
                })
                ->whereDate('orders.created_at', '=', $row)
                ->count();
            $format[] = Carbon::parse($row)->format('d M');
        }

        return json_encode(['date' => $format, 'result' => $result]);
    }

    public static function getByRevenueDay($criteria)
    {
       
        $keyword = $criteria->keyword;
        $filter = $criteria->type;
        $status = $criteria->status;
        
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $date = [];
        $result = [];
        $format = [];

        if ($start_date != '' && $end_date == '') {
            $date[] = $start_date;
        } elseif ($start_date == '' && $end_date != '') {
            $date[] = $end_date;
        } else {
            $start_date = $start_date == '' ? Carbon::now()->subDay(15) : $start_date;
            $end_date = $end_date == '' ? Carbon::now()->addDay() : $end_date->addDay();

            $formated_date = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1D'),
                new \DateTime($end_date)
            );
            foreach ($formated_date as $fd) {
                $date[] = $fd;
            }
            $date = array_reverse($date);
        }
        foreach ($date as $row) {
            $result[] = self::leftJoin('order_details as d', 'orders.id', 'd.order_id')
            ->leftJoin('products as p', 'd.product_id', 'p.id')
            ->where('p.is_otc',1)->paid()->online()->active() 
            ->when($keyword, function ($query) use ($keyword) {
                    return $query->where(function ($q) use ($keyword) {
                        $q->where('billing_first_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_last_name', 'like', "%{$keyword}%")
                            ->orWhere('billing_city', 'like', "%{$keyword}%")
                            ->orWhere('billing_postcode', 'like', "%{$keyword}%")
                            ->orWhere('invoice_no', 'like', "%{$keyword}%")
                            ->orWhere('order_no', 'like', "%{$keyword}%");
                    });
                })
                ->when($status!=null, function ($query) use ($status) {
            
                    return $query->where('orders.order_status', $status);
                })
                ->whereDate('orders.created_at', '=', $row)
                ->sum('orders.paid_amount');
            $format[] = Carbon::parse($row)->format('d M');
        }
        return json_encode(['date' => $format, 'result' => $result]);
    }
    public static function getAll($criteria)
    {
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;
        $status = $criteria->status;
      
        return self::leftJoin('order_details as d', 'orders.id', 'd.order_id')
        ->leftJoin('products as p', 'd.product_id', 'p.id')
        ->where('p.is_otc',1)->paid()->active()
            ->when($keyword, function ($query) use ($keyword) {
                return $query->where(function ($q) use ($keyword) {
                    $q->where('billing_first_name', 'like', "%{$keyword}%")
                        ->orWhere('billing_last_name', 'like', "%{$keyword}%")
                        ->orWhere('billing_city', 'like', "%{$keyword}%")
                        ->orWhere('billing_postcode', 'like', "%{$keyword}%")
                        ->orWhere('invoice_no', 'like', "%{$keyword}%")
                        ->orWhere('order_no', 'like', "%{$keyword}%");
                });
            })
            ->when($start_date == '' && $end_date != '', function ($q) use ($end_date) {
                $q->whereDate('orders.created_at', '=', $end_date);
                return $q;
            })
            ->when($start_date != '' && $end_date == '', function ($q) use ($start_date) {
                $q->whereDate('orders.created_at', '=', $start_date);
                return $q;
            })
            ->when($start_date != '' && $end_date != '', function ($q) use ($start_date, $end_date) {
                $q->whereDate('orders.created_at', '>=', $start_date)
                    ->whereDate('orders.created_at', '<=', $end_date);
                return $q;
            })
            ->when($status!=null, function ($query) use ($status) {
              
                return $query->where('orders.order_status', $status);
            });
    }
}
