<?php

namespace App\Models;

use Carbon\Carbon;
use Carbon\CarbonPeriod;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Product extends Model
{
    /**
     * The categories that belong to the product.
     */
    public function categories()
    {
        return $this->belongsToMany('App\Models\ProductCategory', 'category_product', 'product_id', 'category_id');
    }
    public static function getBySaleDay($criteria)
    {
        $keyword = $criteria->keyword;
        $id = $criteria->id;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;
        $show_bmi = isset($criteria->show_bmi) ? $criteria->show_bmi: false;
        $date = [];
        $result = [];
        $format = [];

        if ($start_date != '' && $end_date == '') {
            $date[] = $start_date;
        } elseif ($start_date == '' && $end_date != '') {
            $date[] = $end_date;
        } else {
            $start_date = $start_date == '' ? Carbon::now()->subDay(15) : $start_date;
            $end_date = $end_date == '' ? Carbon::now()->addDay() : $end_date->addDay();

            $formated_date = new \DatePeriod(
                new \DateTime($start_date),
                new \DateInterval('P1D'),
                new \DateTime($end_date)
            );
            foreach ($formated_date as $fd) {
                $date[] = $fd;
            }
            $date = array_reverse($date);
        }

        foreach ($date as $row) {
            $result[] = self::join('order_details as d', 'd.product_id', 'products.id')
                ->join('orders as o', 'o.id', 'd.order_id')
                ->where('o.payment_status', 0)
                ->where('o.order_status', '!=', 5)
                ->where('o.order_from', 1)
                ->when($keyword, function ($q) use ($keyword) {
                    return $q->where('products.title', 'like', "%{$keyword}%");
                })
                ->when($id, function ($q) use ($id) {
                    return $q->where('products.id', $id);
                })
                ->when($show_bmi == true, function ($q)  {
                    $q->whereHas('categories', function($query) {
                        $query->where('product_categories.id', 103);
                     });
                    return $q;
                })
                ->whereDate('o.created_at', '=', $row)
                ->sum('d.quantity');
            $format[] = Carbon::parse($row)->format('d M');
        }
        return json_encode(['date' => $format, 'result' => $result]);
    }

    public static function getBySalesCount($criteria)
    {
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;
        $id = $criteria->id ?? '';
        $show_bmi = isset($criteria->show_bmi) ? $criteria->show_bmi: false;
        return self::join('order_details as d', 'd.product_id', 'products.id')
            ->join('orders as o', 'o.id', 'd.order_id')
            ->where('o.payment_status', 0)
            ->where('o.order_status', '!=', 5)
            ->where('o.order_from', 1)
            ->when($id, function ($q) use ($id) {
                return $q->where('products.id', $id);
            })
            ->when($keyword, function ($q) use ($keyword) {
                return $q->where('products.title', 'like', "%{$keyword}%");
            })
            ->when($start_date == '' && $end_date != '', function ($q) use ($end_date) {
                $q->whereDate('o.created_at', '=', $end_date);
                return $q;
            })
            ->when($start_date != '' && $end_date == '', function ($q) use ($start_date) {
                $q->whereDate('o.created_at', '=', $start_date);
                return $q;
            })
            ->when($start_date != '' && $end_date != '', function ($q) use ($start_date, $end_date) {
                $q->whereDate('o.created_at', '>=', $start_date)
                    ->whereDate('o.created_at', '<=', $end_date);
                return $q;
            })
            ->when($show_bmi == true, function ($q)  {
                $q->whereHas('categories', function($query) {
                    $query->where('product_categories.id', 103);
                 });
                return $q;
            })
            ->sum('d.quantity');
    }

    public static function getByProductSales($criteria)
    {
        $id = $criteria->id;
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        $filter = $criteria->type;
        $date = [];
        $result = [];
        $format = [];
        if (($start_date == '' && $end_date == '') || $filter == 'all') {
            $check_date = Carbon::now()->format('Y-m-d');
            $date[] = $check_date;
            for ($i = 1; $i <= 14; $i++) {
                $check_date = Carbon::now()->subDays($i)->format('Y-m-d');
                $date[] = $check_date;
            }
        } elseif ($filter == 'today') {
            $check_date = Carbon::now()->format('Y-m-d');
            $date[] = $check_date;
        } else {
            if ($start_date != '' && $end_date == '') {
                $check_date = Carbon::parse($start_date)->format('Y-m-d');
                $date[] = $check_date;
            } elseif ($start_date == '' && $end_date != '') {
                $check_date = Carbon::parse($end_date)->format('Y-m-d');
                $date[] = $check_date;
            } elseif ($start_date != '' && $end_date != '') {
                $period = CarbonPeriod::create($start_date, $end_date);
                foreach ($period as $dates) {
                    $date[] = $dates->format('Y-m-d');
                }
            } else {

            }
        }
        foreach ($date as $row) {
            $result[] = DB::table('orders as o')
                ->join('order_carts as oc', 'o.id', 'oc.order_id')
                ->leftJoin('users as u', 'u.id', 'o.user_id')
                ->join('products as p', 'p.id', 'oc.product_id')
                ->join('contents as c', 'p.content_id', 'c.id')
                ->where('o.payment', 1)
                ->when($id, function ($query) use ($id) {
                    $query->where('c.id', $id);
                    return $query;
                })
                ->when($keyword, function ($q) use ($keyword) {
                    return $q->where(function ($query) use ($keyword) {
                        $query->where('u.name', 'like', "%{$keyword}%")
                            ->orWhere('o.id', 'like', "%{$keyword}%")
                            ->orWhere('u.lname', 'like', "%{$keyword}%")
                            ->orWhere('o.postcode', 'like', "%{$keyword}%");
                    });
                })
                ->whereDate('o.datetime', '=', $row)
                ->sum('oc.qty');
            $format[] = Carbon::parse($row)->format('d M');
        }
        return json_encode(['date' => $format, 'result' => $result]);
    }

    public static function getByProductSalesCount($criteria)
    {
        $id = $criteria->id;
        $keyword = $criteria->keyword;
        $start_date = $criteria->start_date;
        $end_date = $criteria->end_date;
        return DB::table('orders as o')
            ->join('order_carts as oc', 'o.id', 'oc.order_id')
            ->leftJoin('users as u', 'u.id', 'o.user_id')
            ->join('products as p', 'p.id', 'oc.product_id')
            ->join('contents as c', 'p.content_id', 'c.id')
            ->where('o.payment', 1)
            ->when($id, function ($q) use ($id) {
                return $q->where('c.id', $id);
            })
            ->when($keyword, function ($q) use ($keyword) {
                return $q->where(function ($query) use ($keyword) {
                    $query->where('u.name', 'like', "%{$keyword}%")
                        ->orWhere('o.id', 'like', "%{$keyword}%")
                        ->orWhere('u.lname', 'like', "%{$keyword}%")
                        ->orWhere('o.postcode', 'like', "%{$keyword}%");
                });
            })
            ->when($start_date == '' && $end_date != '', function ($q) use ($end_date) {
                return $q->whereDate('o.datetime', '=', $end_date);
            })
            ->when($start_date != '' && $end_date == '', function ($q) use ($start_date) {
                return $q->whereDate('o.datetime', '=', $start_date);
            })
            ->when($start_date != '' && $end_date != '', function ($q) use ($start_date, $end_date) {
                return $q->whereBetween('o.datetime', [$start_date, $end_date]);
            })->sum('oc.qty');
    }

    public static function getMainCategories()
    {
        return DB::table('prdt_cat as pc')
            ->join('contents as c', 'pc.topparent', 'c.id')
            ->groupBy('c.id')
            ->orderBy('c.title', 'asc')
            ->get();
    }

    public static function getOutOfStock()
    {
        return DB::table('products as p')
            ->join('contents as c', 'p.content_id', 'c.id')
            ->select('c.title')
            ->where('stock', '<=', 0)
            ->orderBy('title', 'asc')
            ->limit(5)
            ->get();
    }

    public static function getReorders()
    {
        return DB::table('products as p')
            ->join('contents as c', 'p.content_id', 'c.id')
            ->select('c.title', 'p.stock')
            ->whereColumn('stock', '<=', 'invlimit')
            ->where('stock', '>', 0)
            ->orderBy('stock', 'asc')
            ->limit(5)
            ->get();
    }

}
