@extends('layouts.app')
@section('title',Str::title($page->title))
@section('content')
    <div class="content-wrapper">
        <!-- Content Header (Page header) -->
        <div class="content-header">
            <div class="container-fluid">
                <div class="row mb-2">
                    <div class="col-sm-6">
                        <h1 class="m-0 text-dark">{{ Str::title($page->title) }}</h1>
                    </div><!-- /.col -->
                    <div class="col-sm-6">
                        <ol class="breadcrumb float-sm-right">
                            <li class="breadcrumb-item"><a href="{{ url('/') }}">Home</a></li>
                            <li class="breadcrumb-item active">{{ Str::title($page->title) }}</li>
                        </ol>
                    </div><!-- /.col -->
                </div><!-- /.row -->
            </div><!-- /.container-fluid -->
        </div>
        <!-- /.content-header -->

        <!-- Main content -->
        <section class="content">
            <div class="container-fluid">
                <!-- Info boxes -->
                <div class="row">
                    <div class="col-12 text-right mb-3">
                        <div class="card">
                            <div class="card-body">
                                {!! Form::open(['url' => $page->link, 'method' => 'POST','class'=>'form-horizontal','id'=>'filter-form']) !!}
                                <div class="row">
                                    <label class="col-sm-1 px-0 col-form-label">Filter By: </label>
                                    <div class="col-sm-2">
                                        {!! Form::text('keyword', $request->keyword ?? '', ['class' => 'form-control','id' => 'product-type-head','placeholder'=>'Keyword']) !!}
                                    </div>
                                   
                                    <div class="col-sm-3">
                                        {!! Form::select('sort',$page->sorts, $request->sort ?? '', ['class' => 'form-control','placeholder'=>'Sort By']) !!}
                                    </div>
                                    <div class="col-sm-3">
                                        <div class="checkbox" style="text-align:left">
                                            <label >
                                                <input type="checkbox" {{$request->show_otc?'checked': ''}} name="show_otc"> Show OTC Products Only
                                            </label>
                                        </div>
                                        
                                             <div class="checkbox" style="text-align:left">
                                            <label >
                                                <input type="checkbox" {{$request->show_in_stock?'checked': ''}} name="show_in_stock"> Show Instock Products Only
                                            </label>
                                        </div>
                                        
                                              <div class="checkbox" style="text-align:left">
                                            <label >
                                                <input type="checkbox" {{$request->show_out_stock?'checked': ''}} name="show_out_stock"> Show Out Off stock Products Only
                                            </label>
                                        </div>
                                        
                                        
                                    </div>
                                    
                                    {!! Form::hidden('filter', '', ['id'=>'filter']) !!}
                                    <input type="hidden" name="download_option" id="download-option" value="0" />
                                    <button class="btn btn-danger" id="filter-button"><i class="fas fa-search"></i>
                                    </button>
                                    <button class="btn btn-primary ml-2" type="button" id="download-button"><i class="fas fa-download"></i>
                                    </button>
                                  
                                </div>
                                {!! Form::close() !!}
                            </div>
                        </div>
                    </div>
                </div>
             
                <div class="row">
                    <div class="col-12">
                        <div class="card card-danger card-outline">
                            <div class="card-header">
                                <h3 class="card-title">Product Lists</h3>
                            </div>
                            <!-- /.card-header -->
                            <div class="card-body p-0 data-table-container">
                                <div class="row">
                                    <div class="col-12">
                                        <div class="table-responsive p-3">
                                            <table class="table table-hover table-striped table-bordered data-tables"
                                                   data-url="{{ url('products/all_lists') }}" data-form="filter"
                                                   data-length="25">
                                                <thead>
                                                <tr>
                                                    <th data-orderable="false">Title</th>
                                                    <th data-orderable="false">Stock Quantity</th>
                                                    <th data-orderable="false">Options</th>
                                                </tr>
                                                </thead>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                                <!-- /.table-responsive -->
                            </div>
                        </div>
                    </div>

                    <!-- /.col -->
                </div>
            </div><!--/. container-fluid -->
        </section>
        <!-- /.content -->
    </div>
@endsection
@push('page-js')
    <script src="{{ asset('plugins/chart.js/Chart.min.js') }}"></script>
    <script src="{{ asset('dist/js/pages/products.js') }}"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-3-typeahead/4.0.1/bootstrap3-typeahead.min.js"></script>
    <script>
        $('#download-option').val(0);
        var route = "{{ URL::to('/products/typehead/suggesion') }}";
        $('#product-type-head').typeahead({
            source: function (query, process) {
                return $.get(route, {
                    query: query
                }, function (data) {
                    return process(data);
                });
            }
        });
        $('#download-button').on('click',function(e){
            e.preventDefault();
            $('#download-option').val(1);
            $('#filter-form').trigger('submit');
            setTimeout(() => {
                $('#download-option').val(0);
            }, 500);
        })
    </script>
@endpush
