@extends('layouts.app')
@section('title',Str::title($page->title))
@section('content')
    <div class="content-wrapper">
        <!-- Content Header (Page header) -->
        <div class="content-header">
            <div class="container-fluid">
                <div class="row mb-2">
                    <div class="col-sm-6">
                        <h1 class="m-0 text-dark">{{ Str::title($page->title) }}</h1>
                    </div><!-- /.col -->
                    <div class="col-sm-6">
                        <ol class="breadcrumb float-sm-right">
                            <li class="breadcrumb-item"><a href="{{ url('/') }}">Home</a></li>
                            <li class="breadcrumb-item active">{{ Str::title($page->title) }}</li>
                        </ol>
                    </div><!-- /.col -->
                </div><!-- /.row -->
            </div><!-- /.container-fluid -->
        </div>
        <!-- /.content-header -->

        <!-- Main content -->
        <section class="content">
            <div class="container-fluid">
                <!-- Info boxes -->
                <div class="row">
                    <div class="col-12 text-right mb-3">
                        <div class="card">
                            <div class="card-body">
                                {!! Form::open(['url' => $page->link, 'method' => 'POST','class'=>'form-horizontal','id'=>'filter-form']) !!}
                                <div class="row">
                                    <label class="col-sm-1 px-0 col-form-label">Filter By: </label>
                                    <div class="col-sm-4">
                                        {!! Form::text('keyword', $request->keyword ?? '', ['class' => 'form-control','placeholder'=>'Keyword']) !!}
                                    </div>

                                    <div class="col-sm-3">
                                        {!! Form::select('sort',$page->sorts, $request->sort ?? '', ['class' => 'form-control','placeholder'=>'Sort By']) !!}
                                    </div>
                                    <button class="btn btn-danger" id="filter-button"><i class="fas fa-search"></i>
                                    </button>
                                       <input type="hidden" name="download_option" id="download-option" value="0" />
                                       <button class="btn btn-primary ml-2" type="button" id="download-button"><i class="fas fa-download"></i>
                                    </button>
                                    
                                </div>
                                {!! Form::close() !!}
                            </div>
                        </div>
                    </div>
                </div>

                <div class="row">
                    <div class="col-12">
                        <div class="card card-danger card-outline">
                            <div class="card-header">
                                <h3 class="card-title">Stock Lists</h3>
                            </div>
                            <!-- /.card-header -->
                            <div class="card-body p-0 data-table-container">
                                <div class="row">
                                    <div class="col-12">
                                        <div class="table-responsive p-3">
                                            <table class="table table-hover table-striped table-bordered data-tables"
                                                   data-url="{{ $page->link.'/lists' }}" data-form="filter"
                                                   data-length="25">
                                                <thead>
                                                <tr>
                                                    <th data-orderable="false">Products</th>
                                                    <th data-orderable="false">Stocks</th>
                                                </tr>
                                                </thead>
                                            </table>
                                        </div>
                                    </div>
                                </div>
                                <!-- /.table-responsive -->
                            </div>
                        </div>
                    </div>

                    <!-- /.col -->
                </div>
            </div><!--/. container-fluid -->
        </section>
        <!-- /.content -->
    </div>
@endsection
@push('page-css')
    <link rel="stylesheet" href="{{ asset('plugins/datatables-bs4/css/dataTables.bootstrap4.min.css') }}">
    <link rel="stylesheet" href="{{ asset('plugins/datatables-responsive/css/responsive.bootstrap4.min.css') }}">
    <link rel="stylesheet" href="{{ asset('plugins/bootstrap-datepicker/css/bootstrap-datepicker.css') }}">
@endpush
@push('page-js')
    <script src="{{ asset('plugins/datatables/jquery.dataTables.min.js') }}"></script>
    <script src="{{ asset('plugins/datatables-bs4/js/dataTables.bootstrap4.min.js') }}"></script>
    <script src="{{ asset('plugins/datatables-responsive/js/dataTables.responsive.min.js') }}"></script>
    <script src="{{ asset('plugins/datatables-responsive/js/responsive.bootstrap4.min.js') }}"></script>
    <script src="{{ asset('plugins/moment/moment.min.js') }}"></script>
    <script src="{{ asset('plugins/bootstrap-datepicker/js/bootstrap-datepicker.js') }}"></script>
    <script>
        $('.date-picker').datepicker({
            format: 'dd/mm/yyyy',
            autoclose: true,
            todayHighlight: true
        });

        function changeDates(value) {
            $('#filter').val(value);
            $('#filter-form').submit();
        }
        
             $('#download-button').on('click',function(e){
                 e.preventDefault();
                 $('#download-option').val(1);
                 $('#filter-form').trigger('submit');
                    setTimeout(() => {
                        $('#download-option').val(0);
                    }, 500);
            })
        
    </script>
@endpush
